#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkmeshvectorfromcomponents.py,v $
## Language:  Python
## Date:      $Date: 2005/09/14 09:49:59 $
## Version:   $Revision: 1.6 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import pypes


class vmtkMeshVectorFromComponents(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Mesh = None

        self.VectorArrayName = None
        self.ComponentsArrayNames = None

        self.RemoveComponentArrays = False

        self.SetScriptName('vmtkmeshvectorfromcomponents')
        self.SetScriptDoc('create a vector array from a number of scalar arrays treated as vector components')
        self.SetInputMembers([
            ['Mesh','i','vtkUnstructuredGrid',1,'','the input mesh','vmtkmeshreader'],
            ['VectorArrayName','vector','str',1,'',''],
            ['ComponentsArrayNames','components','str',-1,'',''],
            ['RemoveComponentArrays','removecomponents','bool',1,'','']
            ])
        self.SetOutputMembers([
            ['Mesh','o','vtkUnstructuredGrid',1,'','the output mesh','vmtkmeshwriter']
            ])

    def Execute(self):

        if (self.Mesh == None):
            self.PrintError('Error: no Mesh.')

        numberOfComponents = len(self.ComponentsArrayNames)

        vectorArray = vtk.vtkDoubleArray()
        vectorArray.SetName(self.VectorArrayName)
        vectorArray.SetNumberOfComponents(numberOfComponents)
        vectorArray.SetNumberOfTuples(self.Mesh.GetNumberOfPoints())

        for i in range(numberOfComponents):
            componentArray = self.Mesh.GetPointData().GetArray(self.ComponentsArrayNames[i])
            vectorArray.CopyComponent(i,componentArray,0)
            if self.RemoveComponentArrays:
                self.Mesh.GetPointData().RemoveArray(self.ComponentsArrayNames[i])

        self.Mesh.GetPointData().AddArray(vectorArray)


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program: VMTK
## Language:  Python
## Date:      May 30, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENCE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtkcenterlinesnetwork as centerlinesnetwork


def test_centerline_extraction_surface_with_no_hole(aorta_surface):
    clnetwork = centerlinesnetwork.vmtkCenterlinesNetwork()
    clnetwork.Surface = aorta_surface
    clnetwork.RandomSeed = 42
    clnetwork.Execute()
    centerlines = clnetwork.Centerlines
    assert (centerlines.GetNumberOfPoints() > 100) == True



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkcenterlineviewer.py,v $
## Language:  Python
## Date:      $Date: 2006/05/26 12:35:13 $
## Version:   $Revision: 1.3 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import vtkvmtk
from vmtk import vmtkrenderer
from vmtk import pypes


class vmtkCenterlineViewer(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Centerlines = None
        self.PointDataArrayName = ''
        self.CellDataArrayName = ''
        self.Display = 1
        self.Legend = 1
        self.ColorMap = 'cooltowarm'
        self.NumberOfColors = 256
        self.vmtkRenderer = None
        self.OwnRenderer = 0

        self.SetScriptName('vmtkcenterlineviewer')
        self.SetScriptDoc('display a 3D centerline')
        self.SetInputMembers([
            ['Centerlines','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['PointDataArrayName','pointarray','str',1,''],
            ['CellDataArrayName','cellarray','str',1,''],
            ['Legend','legend','bool',1,''],
            ['ColorMap','colormap','str',1,'["rainbow","blackbody","cooltowarm","grayscale"]','choose the color map'],
            ['NumberOfColors','numberofcolors','int',1,'','number of colors in the color map'],
            ['vmtkRenderer','renderer','vmtkRenderer',1,'','external renderer']])
        self.SetOutputMembers([
            ['Centerlines','o','vtkPolyData',1,'','the output centerlines','vmtksurfacewriter']])

    def Execute(self):

        if not self.Centerlines:
            self.PrintError('Error: No input centerlines.')
            return

        if not self.vmtkRenderer:
            self.vmtkRenderer = vmtkrenderer.vmtkRenderer()
            self.vmtkRenderer.Initialize()
            self.OwnRenderer = 1

        self.vmtkRenderer.RegisterScript(self)

        if self.CellDataArrayName:
            cellCenters = vtk.vtkCellCenters()
            cellCenters.SetInputData(self.Centerlines)
            cellCenters.Update()
            cellCenters.GetOutput().GetPointData().SetActiveScalars(self.CellDataArrayName)
            labelsMapper = vtk.vtkLabeledDataMapper();
            labelsMapper.SetInputConnection(cellCenters.GetOutputPort())
            labelsMapper.SetLabelModeToLabelScalars()
            labelsActor = vtk.vtkActor2D()
            labelsActor.SetMapper(labelsMapper)
            self.vmtkRenderer.Renderer.AddActor(labelsActor)

        centerlineMapper = vtk.vtkPolyDataMapper()
        centerlineMapper.SetInputData(self.Centerlines)
        if self.CellDataArrayName and not self.PointDataArrayName:
            centerlineMapper.ScalarVisibilityOn()
            centerlineMapper.SetScalarModeToUseCellData()
            self.Centerlines.GetCellData().SetActiveScalars(self.CellDataArrayName)
            centerlineMapper.SetScalarRange(self.Centerlines.GetCellData().GetScalars().GetRange(0))
        elif self.PointDataArrayName:
            centerlineMapper.ScalarVisibilityOn()
            centerlineMapper.SetScalarModeToUsePointData()
            self.Centerlines.GetPointData().SetActiveScalars(self.PointDataArrayName)
            centerlineMapper.SetScalarRange(self.Centerlines.GetPointData().GetScalars().GetRange(0))
        else:
            centerlineMapper.ScalarVisibilityOff()

        if self.ColorMap == 'grayscale':
            lut = centerlineMapper.GetLookupTable()
            lut.SetNumberOfTableValues(self.NumberOfColors)
            lut.SetValueRange(0.0,1.0)
            lut.SetSaturationRange(0.0,0.0)
            lut.Build()
            centerlineMapper.SetLookupTable(lut)

        if self.ColorMap == 'rainbow':
            lut = centerlineMapper.GetLookupTable()
            lut.SetHueRange(0.666667,0.0)
            lut.SetSaturationRange(0.75,0.75)
            lut.SetValueRange(1.0,1.0)
            lut.SetAlphaRange(1.0,1.0)
            lut.SetNumberOfColors(self.NumberOfColors)
            lut.Build()
            centerlineMapper.SetLookupTable(lut)

        if self.ColorMap == 'blackbody':
            lut = centerlineMapper.GetLookupTable()
            lut.SetNumberOfTableValues(self.NumberOfColors)
            colorTransferFunction = vtk.vtkColorTransferFunction()
            colorTransferFunction.SetColorSpaceToRGB()
            colorTransferFunction.AddRGBPoint(0,0.0,0.0,0.0)
            colorTransferFunction.AddRGBPoint(0.4,0.901961,0.0,0.0)
            colorTransferFunction.AddRGBPoint(0.8,0.901961,0.901961,0.0)
            colorTransferFunction.AddRGBPoint(1.0,1.0,1.0,1.0)
            for ii,ss in enumerate([float(xx)/float(self.NumberOfColors) for xx in range(self.NumberOfColors)]):
                cc = colorTransferFunction.GetColor(ss)
                lut.SetTableValue(ii,cc[0],cc[1],cc[2],1.0)
            lut.Build()
            centerlineMapper.SetLookupTable(lut)

        if self.ColorMap == 'cooltowarm':
            lut = centerlineMapper.GetLookupTable()
            lut.SetNumberOfTableValues(self.NumberOfColors)
            colorTransferFunction = vtk.vtkColorTransferFunction()
            colorTransferFunction.SetColorSpaceToDiverging()
            colorTransferFunction.AddRGBPoint(0,0.231373,0.298039,0.752941)
            colorTransferFunction.AddRGBPoint(0.5,0.865003,0.865003,0.865003)
            colorTransferFunction.AddRGBPoint(1.0,0.705882,0.0156863,0.14902)
            for ii,ss in enumerate([float(xx)/float(self.NumberOfColors) for xx in range(self.NumberOfColors)]):
                cc = colorTransferFunction.GetColor(ss)
                lut.SetTableValue(ii,cc[0],cc[1],cc[2],1.0)
            lut.Build()
            centerlineMapper.SetLookupTable(lut)

        centerlineActor = vtk.vtkActor()
        centerlineActor.SetMapper(centerlineMapper)
        self.vmtkRenderer.Renderer.AddActor(centerlineActor)

        scalarBarActor = None
        if self.Legend and centerlineActor and self.PointDataArrayName:
            scalarBarActor = vtk.vtkScalarBarActor()
            scalarBarActor.SetLookupTable(centerlineActor.GetMapper().GetLookupTable())
            scalarBarActor.GetLabelTextProperty().ItalicOff()
            scalarBarActor.GetLabelTextProperty().BoldOff()
            scalarBarActor.GetLabelTextProperty().ShadowOff()
            scalarBarActor.SetLabelFormat('%.2f')
            scalarBarActor.SetTitle(self.PointDataArrayName)
            self.vmtkRenderer.Renderer.AddActor(scalarBarActor)

        if self.Display:
            self.vmtkRenderer.Render()

        if self.OwnRenderer:
            self.vmtkRenderer.Deallocate()

#        if self.CellDataArrayName:
#            self.vmtkRenderer.Renderer.RemoveActor(labelsActor)
#
#        if self.Legend and centerlineActor:
#            self.vmtkRenderer.Renderer.RemoveActor(scalarBarActor)
#
#        self.vmtkRenderer.Renderer.RemoveActor(centerlineActor)


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program: VMTK
## Language:  Python
## Date:      February 2, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtkcenterlineinterpolation as centerlineinterpolation


def test_default_params_MaximumInscribedSphereRadius(aorta_centerline, compare_centerlines):
    name = __name__ + "_test_default_params_MaximumInscribedSphereRadius.vtp"
    interpolator = centerlineinterpolation.vmtkCenterlineInterpolation()
    interpolator.Centerlines = aorta_centerline
    interpolator.MaskArrayName = "MaximumInscribedSphereRadius"
    interpolator.Execute()

    assert compare_centerlines(interpolator.Centerlines, name) == True


def test_default_params_EdgeArray(aorta_centerline, compare_centerlines):
    name = __name__ + "_test_default_params_EdgeArray.vtp"
    interpolator = centerlineinterpolation.vmtkCenterlineInterpolation()
    interpolator.Centerlines = aorta_centerline
    interpolator.MaskArrayName = "EdgeArray"
    interpolator.Execute()

    assert compare_centerlines(interpolator.Centerlines, name) == True


def test_default_params_EdgePCoordArray(aorta_centerline, compare_centerlines):
    name = __name__ + "_test_default_params_EdgePCoordArray.vtp"
    interpolator = centerlineinterpolation.vmtkCenterlineInterpolation()
    interpolator.Centerlines = aorta_centerline
    interpolator.MaskArrayName = "EdgePCoordArray"
    interpolator.Execute()

    assert compare_centerlines(interpolator.Centerlines, name) == True



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkcenterlineoffsetattributes.py,v $
## Language:  Python
## Date:      $Date: 2006/04/06 16:46:43 $
## Version:   $Revision: 1.8 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
from vmtk import vtkvmtk
import sys

from vmtk import pypes

from vmtk import vmtkrenderer
from vmtk import vmtkcenterlineviewer


class vmtkCenterlineOffsetAttributes(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Centerlines = None
        self.ReferenceSystems = None

        self.ReferenceGroupId = -1

        self.AbscissasArrayName = 'Abscissas'
        self.NormalsArrayName = 'ParallelTransportNormals'
        self.GroupIdsArrayName = 'GroupIds'
        self.CenterlineIdsArrayName = 'CenterlineIds'
        self.ReferenceSystemsNormalArrayName = 'Normal'

        self.ReplaceAttributes = 1

        self.OffsetAbscissasArrayName = 'OffsetAbscissas'
        self.OffsetNormalsArrayName = 'OffsetNormals'

        self.vmtkRenderer = None
        self.OwnRenderer = 0

        self.Interactive = 0

        self.SetScriptName('vmtkcenterlineoffsetattributes')
        self.SetScriptDoc('offset centerline attributes relative to a bifurcation reference system, in such a way that the abscissa of the closest point the the origin is zero, and the centerline normal at that point coincides with the bifurcation reference system normal')
        self.SetInputMembers([
            ['Centerlines','i','vtkPolyData',1,'','the input split centerlines','vmtksurfacereader'],
            ['ReferenceSystems','referencesystems','vtkPolyData',1,'','bifurcation reference systems','vmtksurfacereader'],
            ['ReferenceGroupId','referencegroupid','int',1,'','group id of the reference system to which attributes have to be offset'],
            ['ReplaceAttributes','replaceattributes','bool',1,'','overwrite the existing attributes'],
              ['AbscissasArrayName','abscissasarray','str',1,'','name of the array where centerline abscissas are stored'],
            ['NormalsArrayName','normalsarray','str',1,'','name of the array where centerline normals are stored'],
            ['GroupIdsArrayName','groupidsarray','str',1,'','name of the array where centerline group ids are stored'],
            ['CenterlineIdsArrayName','centerlineidsarray','str',1,'','name of the array where centerline ids are stored'],
            ['ReferenceSystemsNormalArrayName','referencesystemsnormalarray','str',1,'','name of the array where reference system normals are stored'],
            ['OffsetAbscissasArrayName','offsetabscissasarray','str',1,'','name of the array where offset centerline abscissas have to be stored if ReplaceAttributes is off'],
            ['OffsetNormalsArrayName','offsetnormalsarray','str',1,'','name of the array where offset centerline normals have to be stored if ReplaceAttributes is off'],
                        ['Interactive','interactive','bool',1],
            ['vmtkRenderer','renderer','vmtkRenderer',1,'','external renderer']
            ])
        self.SetOutputMembers([
            ['Centerlines','o','vtkPolyData',1,'','the output centerlines','vmtksurfacewriter'],
            ['ReferenceGroupId','referencegroupid','int',1,'','group id of the reference system to which attributes are offset'],
            ['OffsetAbscissasArrayName','offsetabscissasarray','str',1,'','name of the array where offset centerline abscissas are stored if ReplaceAttributes is off'],
            ['OffsetNormalsArrayName','offsetnormalsarray','str',1,'','name of the array where offset centerline normals are stored if ReplaceAttributes is off'],
            ['AbscissasArrayName','abscissasarray','str',1,'','name of the array where centerline abscissas are stored'],
            ['NormalsArrayName','normalsarray','str',1,'','name of the array where centerline normals are stored']
            ])

    def GroupIdValidator(self,text):
        import string
        if not text:
            return 0
        for char in text:
            if char not in string.digits:
                return 0
        return 1

    def Execute(self):

        if self.Centerlines == None:
            self.PrintError('Error: No input centerlines.')

        if self.ReferenceSystems == None:
            self.PrintError('Error: No input reference systems.')

        if self.Interactive and not self.vmtkRenderer:
            self.vmtkRenderer = vmtkrenderer.vmtkRenderer()
            self.vmtkRenderer.Initialize()
            self.OwnRenderer = 1

        if self.Interactive:

            self.vmtkRenderer.RegisterScript(self)

            viewer = vmtkcenterlineviewer.vmtkCenterlineViewer()
            viewer.Centerlines = self.Centerlines
            viewer.CellDataArrayName = self.GroupIdsArrayName
            viewer.vmtkRenderer = self.vmtkRenderer
            viewer.InputText = self.InputText
            viewer.OutputText = self.OutputText
            viewer.PrintError = self.PrintError
            viewer.PringLog = self.PrintLog
            viewer.Display = 0
            viewer.Execute()

            groupIdString = self.InputText("Please input the reference groupId:\n",self.GroupIdValidator)
            self.ReferenceGroupId = int(groupIdString)

        offsetFilter = vtkvmtk.vtkvmtkCenterlineReferenceSystemAttributesOffset()
        offsetFilter.SetInputData(self.Centerlines)
        offsetFilter.SetReferenceSystems(self.ReferenceSystems)
        offsetFilter.SetAbscissasArrayName(self.AbscissasArrayName)
        offsetFilter.SetNormalsArrayName(self.NormalsArrayName)
        if not self.ReplaceAttributes:
            offsetFilter.SetOffsetAbscissasArrayName(self.OffsetAbscissasArrayName)
            offsetFilter.SetOffsetNormalsArrayName(self.OffsetNormalsArrayName)
        else:
            offsetFilter.SetOffsetAbscissasArrayName(self.AbscissasArrayName)
            offsetFilter.SetOffsetNormalsArrayName(self.NormalsArrayName)
        offsetFilter.SetGroupIdsArrayName(self.GroupIdsArrayName)
        offsetFilter.SetCenterlineIdsArrayName(self.CenterlineIdsArrayName)
        offsetFilter.SetReferenceSystemsNormalArrayName(self.ReferenceSystemsNormalArrayName)
        offsetFilter.SetReferenceSystemsGroupIdsArrayName(self.GroupIdsArrayName)
        offsetFilter.SetReferenceGroupId(self.ReferenceGroupId)
        offsetFilter.Update()

        self.Centerlines = offsetFilter.GetOutput()

        if self.ReferenceGroupId == -1:
            self.ReferenceGroupId = offsetFilter.GetReferenceGroupId()

        if self.OwnRenderer:
            self.vmtkRenderer.Deallocate()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program: VMTK
## Language:  Python
## Date:      January 12, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtksurfaceconnectivity as connectivity
import os


@pytest.fixture(scope='module')
def aorta_surface_two_segments(input_datadir):
    import vmtk.vmtksurfacereader as surfacereader
    reader = surfacereader.vmtkSurfaceReader()
    reader.InputFileName = os.path.join(input_datadir, 'aorta-surface-two-segments.vtp')
    reader.Execute()
    return reader.Surface


def test_extract_largest_surface(aorta_surface_two_segments, compare_surfaces):
    name = __name__ + '_test_extract_largest_surface.vtp'
    connectiv = connectivity.vmtkSurfaceConnectivity()
    connectiv.Surface = aorta_surface_two_segments
    connectiv.Method = 'largest'
    connectiv.CleanOutput = 1
    connectiv.Execute()

    assert compare_surfaces(connectiv.Surface, name) == True


def test_extract_closest_to_reference_surface(aorta_surface_two_segments, aorta_surface_reference, compare_surfaces):
    name = __name__ + '_test_extract_closest_to_reference_surface.vtp'
    connectiv = connectivity.vmtkSurfaceConnectivity()
    connectiv.Surface = aorta_surface_two_segments
    connectiv.Method = 'closest'
    connectiv.ReferenceSurface = aorta_surface_reference
    connectiv.Execute()

    assert compare_surfaces(connectiv.Surface, name) == True


def test_extract_closest_to_point(aorta_surface_two_segments, compare_surfaces):
    name = __name__ + '_test_extract_closest_to_point.vtp'
    connectiv = connectivity.vmtkSurfaceConnectivity()
    connectiv.Surface = aorta_surface_two_segments
    connectiv.Method = 'closest'
    connectiv.ClosestPoint = [0.0, 0.0, 0.0]
    connectiv.Execute()

    assert compare_surfaces(connectiv.Surface, name) == True



from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY

__all__ = [
    'vmtk.vmtkactivetubes',
    'vmtk.vmtkbifurcationprofiles',
    'vmtk.vmtkbifurcationreferencesystems',
    'vmtk.vmtkbifurcationsections',
    'vmtk.vmtkbifurcationvectors',
    'vmtk.vmtkboundarylayer',
    'vmtk.vmtkboundaryreferencesystems',
    'vmtk.vmtkbranchclipper',
    'vmtk.vmtkbranchextractor',
    'vmtk.vmtkbranchgeometry',
    'vmtk.vmtkcenterlineimage',
    'vmtk.vmtkbranchmapping',
    'vmtk.vmtkbranchmetrics',
    'vmtk.vmtkbranchpatching',
    'vmtk.vmtkbranchsections',
    'vmtk.vmtkcenterlineattributes',
    'vmtk.vmtkcenterlinegeometry',
    'vmtk.vmtkcenterlineinterpolation',
    'vmtk.vmtkcenterlinelabeler',
    'vmtk.vmtkcenterlinemerge',
    'vmtk.vmtkcenterlinemodeller',
    'vmtk.vmtkcenterlineoffsetattributes',
    'vmtk.vmtkcenterlineresampling',
    'vmtk.vmtkcenterlines',
    'vmtk.vmtkcenterlinestonumpy',
    'vmtk.vmtkcenterlinesections',
    'vmtk.vmtkcenterlinesmoothing',
    'vmtk.vmtkcenterlinesnetwork',
    'vmtk.vmtkcenterlineviewer',
    'vmtk.vmtkdelaunayvoronoi',
    'vmtk.vmtkdistancetocenterlines',
    'vmtk.vmtkendpointextractor',
    'vmtk.vmtkflowextensions',
    'vmtk.vmtkicpregistration',
    'vmtk.vmtkimagebinarize',
    'vmtk.vmtkimagecast',
    'vmtk.vmtkimagecompose',
    'vmtk.vmtkimagecurvedmpr',
    'vmtk.vmtkimagefeaturecorrection',
    'vmtk.vmtkimagefeatures',
    'vmtk.vmtkimageinitialization',
    'vmtk.vmtkimagemipviewer',
    'vmtk.vmtkimagemorphology',
    'vmtk.vmtkimagenormalize',
    'vmtk.vmtkimageobjectenhancement',
    'vmtk.vmtkimageotsuthresholds',
    'vmtk.vmtkimagereader',
    'vmtk.vmtkimagereslice',
    'vmtk.vmtkimageseeder',
    'vmtk.vmtkimageshiftscale',
    'vmtk.vmtkimagesmoothing',
    'vmtk.vmtkimagetonumpy',
    'vmtk.vmtkimageviewer',
    'vmtk.vmtkimagevesselenhancement',
    'vmtk.vmtkimagevoipainter',
    'vmtk.vmtkimagevoiselector',
    'vmtk.vmtkimagevolumeviewer',
    'vmtk.vmtkimagewriter',
    'vmtk.vmtklevelsetsegmentation',
    'vmtk.vmtklineartoquadratic',
    'vmtk.vmtklineresampling',
    'vmtk.vmtklocalgeometry',
    'vmtk.vmtkmarchingcubes',
    'vmtk.vmtkmesharrayoperation',
    'vmtk.vmtkmeshboundaryinspector',
    'vmtk.vmtkmeshbranchclipper',
    'vmtk.vmtkmeshclipper',
    'vmtk.vmtkmeshconnectivity',
    'vmtk.vmtkmeshcutter',
    'vmtk.vmtkmeshdatareader',
    'vmtk.vmtkmeshextractpointdata',
    'vmtk.vmtkmeshlambda2',
    'vmtk.vmtkmeshlinearize',
    'vmtk.vmtkmeshgenerator',
    'vmtk.vmtkmeshmergetimesteps',
    'vmtk.vmtkmeshpolyballevaluation',
    'vmtk.vmtkmeshprojection',
    'vmtk.vmtkmeshreader',
    'vmtk.vmtkmeshscaling',
    'vmtk.vmtkmeshtetrahedralize',
    'vmtk.vmtkmeshtonumpy',
    'vmtk.vmtkmeshtosurface',
    'vmtk.vmtkmeshtransform',
    'vmtk.vmtkmeshtransformtoras',
    'vmtk.vmtkmeshvectorfromcomponents',
    'vmtk.vmtkmeshviewer',
    'vmtk.vmtkmeshvolume',
    'vmtk.vmtkmeshvorticityhelicity',
    'vmtk.vmtkmeshwallshearrate',
    'vmtk.vmtkmeshwriter',
    'vmtk.vmtknetworkeditor',
    'vmtk.vmtknetworkextraction',
    'vmtk.vmtknetworkwriter',
    'vmtk.vmtknumpyreader',
    'vmtk.vmtknumpytocenterlines',
    'vmtk.vmtknumpytoimage',
    'vmtk.vmtknumpytomesh',
    'vmtk.vmtknumpytosurface',
    'vmtk.vmtknumpywriter',
    'vmtk.vmtkparticletracer',
    'vmtk.vmtkpathlineanimator',
    'vmtk.vmtkpointsplitextractor',
    'vmtk.vmtkpointtransform',
    'vmtk.vmtkpolyballmodeller',
    'vmtk.vmtkpotentialfit',
    'vmtk.vmtkpythonscript',
    'vmtk.vmtkrenderer',
    'vmtk.vmtkrendertoimage',
    'vmtk.vmtkrbfinterpolation',
    'vmtk.vmtksurfaceappend',
    'vmtk.vmtksurfacearraysmoothing',
    'vmtk.vmtksurfacearrayoperation',
    'vmtk.vmtksurfacebooleanoperation',
    'vmtk.vmtksurfacecapper',
    'vmtk.vmtksurfacecelldatatopointdata',
    'vmtk.vmtksurfacecenterlineprojection',
    'vmtk.vmtksurfaceclipper',
    'vmtk.vmtksurfacecliploop',
    'vmtk.vmtksurfaceconnectivity',
    'vmtk.vmtksurfaceconnectivityselector',
    'vmtk.vmtksurfacecurvature',
    'vmtk.vmtksurfacedecimation',
    'vmtk.vmtksurfacedistance',
    'vmtk.vmtksurfaceendclipper',
    'vmtk.vmtksurfacekiteremoval',
    'vmtk.vmtksurfaceloopextraction',
    'vmtk.vmtksurfacemassproperties',
    'vmtk.vmtksurfacemodeller',
    'vmtk.vmtksurfacenormals',
    'vmtk.vmtksurfacepointdatatocelldata',
    'vmtk.vmtksurfacepolyballevaluation',
    'vmtk.vmtksurfaceprojection',
    'vmtk.vmtksurfacereader',
    'vmtk.vmtksurfacereferencesystemtransform',
    'vmtk.vmtksurfaceregiondrawing',
    'vmtk.vmtksurfaceremeshing',
    'vmtk.vmtksurfacescaling',
    'vmtk.vmtksurfacesmoothing',
    'vmtk.vmtksurfacesubdivision',
    'vmtk.vmtksurfacetobinaryimage',
    'vmtk.vmtksurfacetonumpy',
    'vmtk.vmtksurfacetransform',
    'vmtk.vmtksurfacetransforminteractive',
    'vmtk.vmtksurfacetransformtoras',
    'vmtk.vmtksurfacetriangle',
    'vmtk.vmtksurfacetomesh',
    'vmtk.vmtksurfaceviewer',
    'vmtk.vmtksurfacewriter',
    'vmtk.vmtksurfmesh',
    'vmtk.vmtktetgen',
    'vmtk.vmtktetringenerator'
    ]

for item in __all__:
    exec('from '+item+' import *')



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkimagereslice.py,v $
## Language:  Python
## Date:      $Date: 2006/07/17 09:53:14 $
## Version:   $Revision: 1.8 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import pypes


class vmtkImageCast(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Image = None

        self.OutputType = 'float'
        self.ShiftScale = 1
        self.WindowLevel = [0.0,0.0]

        self.SetScriptName('vmtkimagecast')
        self.SetScriptDoc('cast an image to a specified type')
        self.SetInputMembers([
            ['Image','i','vtkImageData',1,'','the input image','vmtkimagereader'],
            ['OutputType','type','str',1,'["float","double","uchar","short"]','the output image type'],
            ['ShiftScale','shiftscale','int',1,'','shift scale values to fit windowlevel'],
            ['WindowLevel','windowlevel','float',2,'','window and level for value mapping']
            ])
        self.SetOutputMembers([
            ['Image','o','vtkImageData',1,'','the output image','vmtkimagewriter']
            ])

    def Execute(self):

        if self.Image == None:
            self.PrintError('Error: No input image.')

        if self.OutputType == 'uchar' and self.ShiftScale:
            shiftScale = vtk.vtkImageShiftScale()
            shiftScale.SetInputData(self.Image)
            if self.WindowLevel[0] == 0.0:
                scalarRange = self.Image.GetScalarRange()
                scale = 255.0
                if (scalarRange[1]-scalarRange[0]) > 0.0:
                    scale = 255.0/(scalarRange[1]-scalarRange[0])
                shiftScale.SetShift(-scalarRange[0])
                shiftScale.SetScale(scale)
            else:
                shiftScale.SetShift(-(self.WindowLevel[1]-self.WindowLevel[0]/2.0))
                shiftScale.SetScale(255.0/self.WindowLevel[0])
            shiftScale.SetOutputScalarTypeToUnsignedChar()
            shiftScale.ClampOverflowOn()
            shiftScale.Update()
            self.Image = shiftScale.GetOutput()
        else:
            cast = vtk.vtkImageCast()
            cast.SetInputData(self.Image)
            if self.OutputType == 'float':
                cast.SetOutputScalarTypeToFloat()
            elif self.OutputType == 'double':
                cast.SetOutputScalarTypeToDouble()
            elif self.OutputType == 'uchar':
                cast.SetOutputScalarTypeToUnsignedChar()
            elif self.OutputType == 'short':
                cast.SetOutputScalarTypeToShort()
            cast.Update()
            self.Image = cast.GetOutput()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkpotentialfit.py,v $
## Language:  Python
## Date:      $Date: 2005/09/14 09:51:28 $
## Version:   $Revision: 1.5 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
from vmtk import vtkvmtk
import sys

from vmtk import pypes


class vmtkPotentialFit(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.Image = None

        self.PotentialImage = None

        self.NumberOfIterations = 100
        self.NumberOfStiffnessSubIterations = 0
        self.NumberOfInflationSubIterations = 0
        self.PotentialWeight = 1.0
        self.StiffnessWeight = 1.0
        self.InflationWeight = 1.0
        self.Convergence = 1E-5
        self.MaxTimeStep = 1.0
        self.TimeStep = 1.0
        self.AdaptiveTimeStep = 1
        self.Relaxation = 1E-1
        self.Dimensionality = 3

        self.SetScriptName('vmtkpotentialfit')
        self.SetScriptDoc('explicitly deformable model which evolves a surface to gradient magnitudes of an input image')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','','vmtksurfacereader'],
            ['Image','image','vtkImageData',1,'','','vmtkimagereader'],
            ['NumberOfIterations','iterations','int',1,'(0,)'],
            ['NumberOfStiffnessSubIterations','stiffnesssubiterations','int',1,'(0,)'],
            ['NumberOfInflationSubIterations','inflationsubiterations','int',1,'(0,)'],
            ['PotentialWeight','potentialweight','float',1,'(0.0,)'],
            ['StiffnessWeight','stiffnessweight','float',1,'(0.0,)'],
            ['InflationWeight','inflationweight','float',1,'(0.0,)'],
            ['Convergence','convergence','float',1,'(0.0,)'],
            ['MaxTimeStep','maxtimestep','float',1,'(0.0,)'],
            ['TimeStep','timestep','float',1,'(0.0,)'],
            ['AdaptiveTimeStep','adaptivetimestep','bool',1],
            ['Relaxation','relaxation','float',1,'(0.0,)'],
            ['Dimensionality','dimensionality','int',1,'(2,3)']
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','','vmtksurfacewriter']
            ])

    def Execute(self):

        if (self.Image == None):
            self.PrintError('Error: Image not set.')

        polyDataPotentialFit = vtkvmtk.vtkvmtkPolyDataPotentialFit()
        polyDataPotentialFit.SetInputData(self.Surface)
        polyDataPotentialFit.SetPotentialImage(self.Image)
        polyDataPotentialFit.SetMaxTimeStep(self.MaxTimeStep)
        polyDataPotentialFit.SetTimeStep(self.TimeStep)
        polyDataPotentialFit.SetAdaptiveTimeStep(self.AdaptiveTimeStep)
        polyDataPotentialFit.SetRelaxation(self.Relaxation)
        polyDataPotentialFit.SetConvergence(self.Convergence)
        polyDataPotentialFit.SetPotentialWeight(self.PotentialWeight)
        polyDataPotentialFit.SetStiffnessWeight(self.StiffnessWeight)
        polyDataPotentialFit.SetInflationWeight(self.InflationWeight)
        polyDataPotentialFit.SetNumberOfStiffnessSubIterations(self.NumberOfStiffnessSubIterations)
        polyDataPotentialFit.SetNumberOfInflationSubIterations(self.NumberOfInflationSubIterations)
        polyDataPotentialFit.SetNumberOfIterations(self.NumberOfIterations)
        polyDataPotentialFit.SetDimensionality(self.Dimensionality)
        polyDataPotentialFit.Update()

        self.Surface = polyDataPotentialFit.GetOutput()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY

__all__ = [
    'vmtk.pypescript',
    'vmtk.pype',
    'vmtk.pypebatch'
    ]

for item in __all__:
    exec('from '+item+' import *')



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtksurfacedecimation.py,v $
## Language:  Python
## Date:      $Date: 2006/02/23 09:27:52 $
## Version:   $Revision: 1.7 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import pypes


class vmtkSurfaceDecimation(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None

        self.TargetReduction = 0.5
        self.BoundaryVertexDeletion = 0

        self.SetScriptName('vmtksurfacedecimation')
        self.SetScriptDoc('reduce the number of triangles in a surface')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['TargetReduction','reduction','float',1,'(0.0,1.0)','desired number of triangles relative to input number of triangles'],
            ['BoundaryVertexDeletion','boundarydeletion','bool',1,'','toggle allow boundary point deletion']
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','the output surface','vmtksurfacewriter']
            ])

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: No input surface.')

        triangleFilter = vtk.vtkTriangleFilter()
        triangleFilter.SetInputData(self.Surface)
        triangleFilter.Update()

        decimationFilter = vtk.vtkDecimatePro()
        decimationFilter.SetInputConnection(triangleFilter.GetOutputPort())
        decimationFilter.SetTargetReduction(self.TargetReduction)
        decimationFilter.SetBoundaryVertexDeletion(self.BoundaryVertexDeletion)
        decimationFilter.PreserveTopologyOn()
        decimationFilter.Update()

        cleaner = vtk.vtkCleanPolyData()
        cleaner.SetInputConnection(decimationFilter.GetOutputPort())
        cleaner.Update()

        triangleFilter = vtk.vtkTriangleFilter()
        triangleFilter.SetInputConnection(cleaner.GetOutputPort())
        triangleFilter.Update()

        self.Surface = triangleFilter.GetOutput()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



"""vmtk - the Vascular Modeling Toolkit.

This module is installed as vmtk/__init__.py in the Python wheels built by
the top-level setup.py (see the VMTK_WHEEL_BUILD option in the top-level
CMakeLists.txt). Regular (non-wheel) installs use the empty __init__.py.in
instead.

In the wheel layout the vmtk package directory contains both the wrapped
Python extension modules and the vmtk shared libraries, while the VTK
libraries live in the sibling "vtkmodules" package of the "vtk" wheel.
Since Python 3.8, Windows extension modules resolve dependent DLLs only
from directories registered with os.add_dll_directory(), so register both
directories here before any wrapped module is imported. Linux and macOS
need no equivalent: the libraries are built with relative rpaths
($ORIGIN / @loader_path) that resolve the same locations.
"""

import os as _os
import sys as _sys

if _sys.platform == "win32" and hasattr(_os, "add_dll_directory"):
    _package_dir = _os.path.dirname(_os.path.abspath(__file__))
    _vtkmodules_dir = _os.path.join(_os.path.dirname(_package_dir), "vtkmodules")
    for _dll_dir in (_package_dir, _vtkmodules_dir):
        if _os.path.isdir(_dll_dir):
            _os.add_dll_directory(_dll_dir)
    del _package_dir, _vtkmodules_dir, _dll_dir

del _os, _sys



## Program: VMTK
## Language:  Python
## Date:      January 10, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import sys

import pytest
import vmtk.vmtkimagefeatures as imagefeatures


#TODO: WHY DOES "fwhm" seg fault?
@pytest.mark.parametrize("featureType,paramid", [
    ("vtkgradient", '0'),
    ("gradient", '1'),
])
def test_features_types(aorta_image, compare_images, featureType, paramid, write_image):
    name = __name__ + '_test_features_types_' + paramid + '.mha'
    featurer = imagefeatures.vmtkImageFeatures()
    featurer.Image = aorta_image
    featurer.FeatureImageType = featureType
    featurer.Execute()

    assert compare_images(featurer.Image, name) == True


@pytest.mark.parametrize("featureType,paramid", [
    ("upwind", '2'),
])
def test_features_types_upwind(aorta_image, compare_images, featureType, paramid, write_image):
    name = __name__ + '_test_features_types_' + paramid + '.mha'
    featurer = imagefeatures.vmtkImageFeatures()
    featurer.Image = aorta_image
    featurer.FeatureImageType = featureType
    featurer.Execute()

    assert compare_images(featurer.Image, name) == True


@pytest.mark.parametrize("featureType,paramid", [
    ("gradient", '0'),
])
def test_sigmoid_on_for_gradient(aorta_image, compare_images, featureType, paramid, write_image):
    name = __name__ + '_test_sigmoid_on_for_gradient_and_upwind_' + paramid + '.mha'
    featurer = imagefeatures.vmtkImageFeatures()
    featurer.Image = aorta_image
    featurer.FeatureImageType = featureType
    featurer.SigmoidRemapping = 1
    featurer.Execute()

    assert compare_images(featurer.Image, name) == True


@pytest.mark.parametrize("featureType,paramid", [
    ("upwind", '1'),
])
def test_sigmoid_on_for_upwind(aorta_image, compare_images, featureType, paramid, write_image):
    name = __name__ + '_test_sigmoid_on_for_gradient_and_upwind_' + paramid + '.mha'
    featurer = imagefeatures.vmtkImageFeatures()
    featurer.Image = aorta_image
    featurer.FeatureImageType = featureType
    featurer.SigmoidRemapping = 1
    featurer.Execute()

    assert compare_images(featurer.Image, name) == True


@pytest.mark.parametrize("derivativeValue,paramid", [
    (0.0, '0'),
    (0.5, '1'),
    (4.8, '2'),
])
def test_derivative_sigma_values_for_gradient(aorta_image, compare_images, derivativeValue, paramid, write_image):
    name = __name__ + '_test_derivative_sigma_values_for_gradient_' + paramid + '.mha'
    featurer = imagefeatures.vmtkImageFeatures()
    featurer.Image = aorta_image
    featurer.FeatureImageType = 'gradient'
    featurer.DerivativeSigma = derivativeValue
    featurer.Execute()

    assert compare_images(featurer.Image, name) == True


# The upwind gradient scheme selects a forward or backward difference per voxel
# based on the sign of a computed quantity. At voxels where that quantity is near
# zero the choice flips on last-bit floating-point differences, which produces a
# full-scale jump in the feature there. On macOS this happens at a few voxels
# (the difference reaches ~0.99 of the [0, 1] range), so the max-absolute-
# difference image comparison cannot pass without a tolerance so large it would
# make the test vacuous. The default-factor upwind test is unaffected; only these
# non-default factors expose the tie-break, so skip just this case on macOS.
@pytest.mark.skipif(sys.platform == 'darwin',
                    reason='upwind tie-break causes isolated full-scale voxel '
                           'differences on macOS; not meaningfully comparable')
@pytest.mark.parametrize("upwindValue,paramid", [
    (0.0, '0'),
    (0.3, '1'),
])
def test_upwind_factor_values_for_upwind(aorta_image, compare_images, upwindValue, paramid, write_image):
    name = __name__ + '_test_upwind_factor_values_for_upwind_' + paramid + '.mha'
    featurer = imagefeatures.vmtkImageFeatures()
    featurer.Image = aorta_image
    featurer.FeatureImageType = 'upwind'
    featurer.UpwindFactor = upwindValue
    featurer.Execute()

    assert compare_images(featurer.Image, name) == True

#TODO: Add test for dimensionality



#!/usr/bin/env python

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import sys
import math
import numpy
import vtk
from vmtk import pypes
from vmtk import vmtkscripts


class VmtkSurfaceExtractAnnularWalls(pypes.pypeScript):
    def __init__(self):
        pypes.pypeScript.__init__(self)

        self.SetScriptName('vmtksurfaceextractannularwalls')
        self.SetScriptDoc('Extract wall surfaces from an annular-cylindric surface.')

        # Define members
        self.Surface = None

        self.DoubleSurface = None
        self.ColoredSurface = None
        self.InnerSurface = None
        self.OuterSurface = None

        self.CellEntityIdsArrayName = 'CellEntityIds'
        self.EndcapsThresholdLow = 0
        self.EndcapsThresholdHigh = 1

        # Potential output values
        self.InnerRegionId = None
        self.OuterRegionId = None

        # Member info: name, cmdlinename, typename, num, default, desc[, defaultpipetoscript]
        self.SetInputMembers([
                ['Surface', 'i', 'vtkPolyData', 1, '',
                 'the input surface', 'vmtksurfacereader'],
                ['CellEntityIdsArrayName', 'entityidsarray', 'str', 1, '',
                 'name of the array where entity ids have been stored'],
                ['EndcapsThresholdLow', 'lowthreshold', 'int', 1, '',
                 'lower threshold for encaps filtering', ''],
                ['EndcapsThresholdHigh', 'highthreshold', 'int', 1, '',
                 'higher threshold for encaps filtering', ''],
                ])
        self.SetOutputMembers([
                ['DoubleSurface', 'doublesurface', 'vtkPolyData', 1, '',
                 'the double surface without caps', 'vmtksurfacewriter'],
                ['ColoredSurface', 'o', 'vtkPolyData', 1, '',
                 'the colored surface', 'vmtksurfacewriter'],
                ['InnerSurface', 'innersurface', 'vtkPolyData', 1, '',
                 'the innermost surface', 'vmtksurfacewriter'],
                ['OuterSurface', 'outersurface', 'vtkPolyData', 1, '',
                 'the outermost surface', 'vmtksurfacewriter'],
                ['CellEntityIdsArrayName', 'entityidsarray', 'str', 1, '',
                 'name of the array where entity ids have been stored'],
                ])

    def Execute(self):
        if self.Surface == None:
            self.PrintError('Error: No Surface.')

        self.removeEndCaps()
        self.colorSurfaceRegions()
        self.extractSurfaces()

    def removeEndCaps(self):
        self.PrintLog("Using thresholding to remove endcaps.")

        th = vtk.vtkThreshold()
        th.SetInputData(self.Surface)
        th.SetInputArrayToProcess(0, 0, 0, 1, self.CellEntityIdsArrayName)
        th.ThresholdBetween(self.EndcapsThresholdLow, self.EndcapsThresholdHigh)
        th.Update()

        gf = vtk.vtkGeometryFilter()
        gf.SetInputConnection(th.GetOutputPort())
        gf.Update()

        self.DoubleSurface = gf.GetOutput()

    def colorSurfaceRegions(self):
        self.PrintLog("Coloring surface regions.")

        connectivityFilter = vtk.vtkPolyDataConnectivityFilter()
        connectivityFilter.SetInputData(self.DoubleSurface)
        connectivityFilter.ColorRegionsOn()
        connectivityFilter.SetExtractionModeToAllRegions()
        connectivityFilter.Update()

        assert connectivityFilter.GetNumberOfExtractedRegions() == 2

        self.ColoredSurface = connectivityFilter.GetOutput()

    def extractSurfaces(self):
        self.PrintLog("Extracting surfaces.")

        def bnorm(data):
            bounds = data.GetBounds()
            return math.sqrt(sum((bounds[2*i+1]-bounds[2*i])**2 for i in range(3)))

        # Get bounds of entire surface
        bounds_norm = bnorm(self.Surface)

        # Currently assuming that this is the region numbers that were produced by coloring!
        # TODO: If necessary, get from array to be more robust.
        region_ids = (0, 1)

        # Extract each surface in turn to find the smallest one
        subsurfaces = {}
        for k in region_ids:
            connectivityFilter = vtk.vtkPolyDataConnectivityFilter()
            connectivityFilter.SetInputData(self.ColoredSurface)
            connectivityFilter.SetExtractionModeToSpecifiedRegions()
            connectivityFilter.AddSpecifiedRegion(k)
            connectivityFilter.ColorRegionsOff()
            connectivityFilter.SetScalarConnectivity(0)
            connectivityFilter.Update()
            subsurfaces[k] = connectivityFilter.GetOutput()

        # The inner surface has smaller bounds
        if bnorm(subsurfaces[region_ids[0]]) < bnorm(subsurfaces[region_ids[1]]):
            self.InnerRegionId = region_ids[0]
            self.OuterRegionId = region_ids[1]
        else:
            self.InnerRegionId = region_ids[1]
            self.OuterRegionId = region_ids[0]
        self.InnerSurface = subsurfaces[self.InnerRegionId]
        self.OuterSurface = subsurfaces[self.OuterRegionId]


if __name__ == '__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtknetworkeditor.py,v $
## Language:  Python
## Date:      $Date: 2006/05/26 12:35:13 $
## Version:   $Revision: 1.3 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

##   Developed with support from the EC FP7/2007-2013: ARCH, Project n. 224390

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import vtkvmtk
from vmtk import vmtkrenderer
from vmtk import vmtkimageviewer
from vmtk import vmtkimagefeatures
from vmtk import pypes

from vmtk import vmtkactivetubes


class vmtkNetworkEditor(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Network = None
        self.RadiusArrayName = 'Radius'
        self.LabelsArrayName = 'Labels'

        self.SplineInterpolation = 1

        self.UseActiveTubes = 0
        self.NumberOfIterations = 100
        self.PotentialWeight = 1.0
        self.StiffnessWeight = 0.0

        self.NetworkTube = None

        self.Selection = None
        self.SelectionPoints = None
        self.SelectionRadiusArray = None
        self.SelectionActor = None
        self.SelectedCellId = -1
        self.SelectedSubId = -1
        self.SelectedPCoords = 0.0

        self.ActiveSegment = None
        self.ActiveSegmentPoints = None
        self.ActiveSegmentCellArray = None
        self.ActiveSegmentRadiusArray = None
        self.ActiveSegmentActor = None

        self.ActiveSegmentSeeds = None
        self.ActiveSegmentSeedsPoints = None
        self.ActiveSegmentSeedsRadiusArray = None
        self.ActiveSegmentSeedsActor = None

        self.AttachedCellIds = [-1,-1]
        self.AttachedSubIds = [-1,-1]
        self.AttachedPCoords = [0.0,0.0]

        self.PlaneWidgetX = None
        self.PlaneWidgetY = None
        self.PlaneWidgetZ = None

        self.Image = None
        self.FeatureImage = None

        self.CurrentRadius = 0.0

        self.NetworkRadiusArray = None
        self.NetworkLabelsArray = None

        self.CellPicker = None

        self.CellIdsToMerge = []

        self.OperationMode = None
        self.PickMode = 'image'

        self.vmtkRenderer = None
        self.OwnRenderer = 0

        self.SetScriptName('vmtknetworkeditor')
        self.SetScriptDoc('extract an approximated centerline from a surface containing at least one hole')
        self.SetInputMembers([
            ['Network','i','vtkPolyData',1,'','the input network','vmtksurfacereader'],
            ['RadiusArrayName','radiusarray','str',1,''],
            ['LabelsArrayName','labelsarray','str',1,''],
            ['SplineInterpolation','spline','bool',1,''],
            ['UseActiveTubes','activetubes','bool',1,''],
            ['NumberOfIterations','iterations','int',1,'(0,)'],
            ['PotentialWeight','potentialweight','float',1,'(0.0,)'],
            ['StiffnessWeight','stiffnessweight','float',1,'(0.0,)'],
            ['PlaneWidgetX','xplane','vtkImagePlaneWidget',1,'','the X image plane widget'],
            ['PlaneWidgetY','yplane','vtkImagePlaneWidget',1,'','the Y image plane widget'],
            ['PlaneWidgetZ','zplane','vtkImagePlaneWidget',1,'','the Z image plane widget'],
            ['Image','image','vtkImageData',1,'','','vmtkimagereader'],
            ['FeatureImage','featureimage','vtkImageData',1,'','','vmtkimagereader'],
            ['vmtkRenderer','renderer','vmtkRenderer',1,'','external renderer']])
        self.SetOutputMembers([
            ['Network','o','vtkPolyData',1,'','the output network','vmtknetworkwriter'],
            ['RadiusArrayName','oradiusarray','str',1,''],
            ['Surface','osurface','vtkPolyData',1,'','the output surface','vmtksurfacewriter']])

    def TogglePickMode(self):
        if self.PickMode == 'image':
            self.PickMode = 'network'
        else:
            self.PickMode = 'image'

    def TogglePlaneWidget(self,planeWidget):
        if not planeWidget:
            return
        interaction = planeWidget.GetInteraction()
        if interaction:
            planeWidget.InteractionOff()
        else:
            planeWidget.InteractionOn()

    def UpdateLabels(self):
        self.CellCenters.Modified()
        self.CellCenters.Update()
        self.Render()

    def ToggleLabels(self):
        if self.LabelsActor.GetVisibility() == 1:
            self.LabelsActor.VisibilityOff()
        else:
            self.LabelsActor.VisibilityOn()
        self.Render()

    def SetPickMode(self,pickMode):
        if pickMode == 'image':
            self.PickMode = 'image'
            if self.PlaneWidgetX:
                self.PlaneWidgetX.InteractionOn()
            if self.PlaneWidgetY:
                self.PlaneWidgetY.InteractionOn()
            if self.PlaneWidgetZ:
                self.PlaneWidgetZ.InteractionOn()
        elif pickMode == 'network':
            self.PickMode = 'network'
            if self.PlaneWidgetX:
                self.PlaneWidgetX.InteractionOff()
            if self.PlaneWidgetY:
                self.PlaneWidgetY.InteractionOff()
            if self.PlaneWidgetZ:
                self.PlaneWidgetZ.InteractionOff()

    def KeyReleaseCallback(self,obj,event):
        pass
        #key = object.GetKeySym()
        #if self.OperationMode == 'add':
        #    if key == 'Shift_L' or key == 'Shift_R':
        #        if self.PickMode == 'network':
        #            self.SetPickMode('image')

    def CheckMenu(self):
        self.vmtkRenderer.RemoveKeyBinding('space')
        self.vmtkRenderer.RemoveKeyBinding('c')
        self.vmtkRenderer.RemoveKeyBinding('u')
        self.vmtkRenderer.RemoveKeyBinding('+')
        self.vmtkRenderer.RemoveKeyBinding('-')
        self.vmtkRenderer.RemoveKeyBinding('=')
        self.vmtkRenderer.RemoveKeyBinding('Return')

    def ShowLabelCallback(self, obj):
        self.ToggleLabels()

    def AddCallback(self, obj):
        self.InputInfo('Switched to add mode.\nCtrl + left click to add tubes.')
        #self.PrintLog('Add mode')
        self.OperationMode = 'add'
        self.InitializeActiveSegment()
        self.vmtkRenderer.AddKeyBinding('space','Toggle image/tube interaction',self.SpaceCallback, '2')
        self.vmtkRenderer.AddKeyBinding('c','Cancel',self.CancelCallback,'2')
        self.vmtkRenderer.AddKeyBinding('u','Undo',self.UndoCallback,'2')
        self.vmtkRenderer.AddKeyBinding('+','Increase radius',self.PlusCallback,'2')
        self.vmtkRenderer.AddKeyBinding('=','Increase radius',self.PlusCallback,'2')
        self.vmtkRenderer.AddKeyBinding('-','Decrease radius',self.MinusCallback,'2')
        self.vmtkRenderer.AddKeyBinding('Return','Accept tube',self.ReturnAddCallback,'2')
        self.UpdateAndRender()

    def DeleteCallback(self, obj):
        self.InputInfo('Switched to delete mode.\nCtrl + left click to delete tubes.')
        #self.PrintLog('Delete mode')
        self.OperationMode = 'delete'
        self.InitializeActiveSegment()
        self.CheckMenu()
        if self.PickMode == 'image':
            self.SetPickMode('network')
            self.InitializeActiveSegment()
        self.UpdateAndRender()

    def SplitCallback(self, obj):
        self.InputInfo('Switched to split mode.\nCtrl + left click to split tubes.')
        #self.PrintLog('Split mode')
        self.OperationMode = 'split'
        self.InitializeActiveSegment()
        self.CheckMenu()
        if self.PickMode == 'image':
            self.SetPickMode('network')
            self.InitializeActiveSegment()
        self.UpdateAndRender()

    def LabelCallback(self, obj):
        #self.PrintLog('Label mode')
        self.InputInfo('Switched to label mode.\nCtrl + left click to add label.')
        self.OperationMode = 'label'
        self.InitializeActiveSegment()
        self.CheckMenu()
        if self.PickMode == 'image':
            self.SetPickMode('network')
            self.InitializeActiveSegment()
        self.UpdateAndRender()

    def MergeCallback(self, obj):
        self.InputInfo('Switched to merge mode.\nCtrl + left click to select two tubes to merge.')
        #self.PrintLog('Merge mode')
        self.OperationMode = 'merge'
        self.InitializeActiveSegment()
        self.CheckMenu()
        if self.PickMode == 'image':
            self.SetPickMode('network')
            self.InitializeActiveSegment()
            self.CellIdsToMerge = []
        self.vmtkRenderer.AddKeyBinding('Return','Accept merge',self.ReturnCallback,'2')
        self.UpdateAndRender()

    def SpaceCallback(self, obj):
        self.TogglePickMode()
        self.TogglePlaneWidget(self.PlaneWidgetX)
        self.TogglePlaneWidget(self.PlaneWidgetY)
        self.TogglePlaneWidget(self.PlaneWidgetZ)
        self.InputInfo('Ctrl + left click to add seeds.')

    def CancelCallback(self, obj):
        self.InitializeActiveSegment()

    def UndoCallback(self, obj):
        numberOfSeeds = self.ActiveSegmentSeedsPoints.GetNumberOfPoints()
        if numberOfSeeds > 0:
            self.ActiveSegmentSeedsPoints.SetNumberOfPoints(numberOfSeeds-1)
            self.ActiveSegmentSeedsRadiusArray.SetNumberOfValues(numberOfSeeds-1)
            self.ActiveSegmentSeeds.Modified()
            self.AttachedCellIds[1] = -1
            self.AttachedSubIds[1] = -1
            self.AttachedPCoords[1] = 0.0
        else:
            self.AttachedCellIds[0] = -1
            self.AttachedSubIds[0] = -1
            self.AttachedPCoords[0] = 0.0
        numberOfPoints = self.ActiveSegmentPoints.GetNumberOfPoints()
        if numberOfPoints > 2:
            self.ActiveSegmentPoints.SetNumberOfPoints(numberOfPoints-1)
            self.ActiveSegmentRadiusArray.SetNumberOfValues(numberOfPoints-1)
            self.ActiveSegmentCellArray.Initialize()
            self.ActiveSegmentCellArray.InsertNextCell(numberOfPoints-1)
            for i in range(numberOfPoints-1):
                self.ActiveSegmentCellArray.InsertCellPoint(i)
            self.ActiveSegment.Modified()
        else:
            self.ActiveSegmentPoints.SetNumberOfPoints(0)
            self.ActiveSegmentCellArray.Initialize()
            self.ActiveSegmentRadiusArray.SetNumberOfValues(0)
            self.ActiveSegment.Modified()
        self.UpdateAndRender()

    def PlusCallback(self, obj):
        numberOfSeeds = self.ActiveSegmentSeedsPoints.GetNumberOfPoints()
        if numberOfSeeds > 0:
            radius = self.ActiveSegmentSeedsRadiusArray.GetValue(numberOfSeeds-1)
            radius += 0.1
            self.ActiveSegmentSeedsRadiusArray.SetValue(numberOfSeeds-1,radius)
            self.CurrentRadius = radius
            self.ActiveSegmentSeeds.Modified()
        numberOfPoints = self.ActiveSegmentPoints.GetNumberOfPoints()
        if numberOfPoints > 0:
            radius = self.ActiveSegmentRadiusArray.GetValue(numberOfPoints-1)
            radius += 0.1
            self.ActiveSegmentRadiusArray.SetValue(numberOfPoints-1,radius)
            self.ActiveSegment.Modified()
        self.UpdateAndRender()

    def MinusCallback(self, obj):
        numberOfSeeds = self.ActiveSegmentSeedsPoints.GetNumberOfPoints()
        if numberOfSeeds > 0:
            radius = self.ActiveSegmentSeedsRadiusArray.GetValue(numberOfSeeds-1)
            radius -= 0.1
            if radius > 0.0:
                self.ActiveSegmentSeedsRadiusArray.SetValue(numberOfSeeds-1,radius)
                self.CurrentRadius = radius
                self.ActiveSegmentSeeds.Modified()
        numberOfPoints = self.ActiveSegmentPoints.GetNumberOfPoints()
        if numberOfPoints > 0:
            radius = self.ActiveSegmentRadiusArray.GetValue(numberOfPoints-1)
            radius -= 0.1
            if radius > 0.0:
                self.ActiveSegmentRadiusArray.SetValue(numberOfPoints-1,radius)
                self.ActiveSegment.Modified()
        self.UpdateAndRender()

    def ReturnAddCallback(self, obj):
        attachedPointId0 = -1
        attachedPointId1 = -1
        cellIdsToRemove = []
        if self.AttachedCellIds[0] != -1:
            attachedPointId0, hasSplit = self.SplitCellNoRemove(self.AttachedCellIds[0],self.AttachedSubIds[0],self.AttachedPCoords[0])
            if hasSplit:
                cellIdsToRemove.append(self.AttachedCellIds[0])
        if self.AttachedCellIds[1] != -1:
            attachedPointId1, hasSplit = self.SplitCellNoRemove(self.AttachedCellIds[1],self.AttachedSubIds[1],self.AttachedPCoords[1])
            if hasSplit:
                cellIdsToRemove.append(self.AttachedCellIds[1])
        if cellIdsToRemove:
            self.RemoveCells(cellIdsToRemove)
        segment = self.ActiveSegmentActor.GetMapper().GetInput()
        if self.UseActiveTubes:
            self.RunActiveTube(segment)
        segmentRadiusArray = segment.GetPointData().GetArray(self.RadiusArrayName)
        numberOfSegmentPoints = segment.GetNumberOfPoints()
        networkPoints = self.Network.GetPoints()
        networkCellArray = self.Network.GetLines()
        cellId = networkCellArray.InsertNextCell(numberOfSegmentPoints)
        for i in range(numberOfSegmentPoints):
            id = -1
            if i == 0 and attachedPointId0 != -1:
                id = attachedPointId0
            elif i == numberOfSegmentPoints-1 and attachedPointId1 != -1:
                id = attachedPointId1
            else:
                id = networkPoints.InsertNextPoint(segment.GetPoint(i))
                self.NetworkRadiusArray.InsertTuple1(id,segmentRadiusArray.GetTuple1(i))
            networkCellArray.InsertCellPoint(id)
        self.NetworkLabelsArray.InsertValue(cellId,'')
        self.Network.BuildCells()
        self.Network.Modified()
        self.NetworkTube.Modified()
        self.UpdateLabels()
        #if self.Image and self.PickMode == 'network':
        #    self.SetPickMode('image')
        self.InitializeSelection()
        self.InitializeActiveSegment()

    def ReturnCallback(self, obj):
        numberOfActiveCells = self.ActiveSegment.GetNumberOfCells()
        if numberOfActiveCells != 2 or len(self.CellIdsToMerge) != 2:
            return
        cell0ToMerge = self.Network.GetCell(self.CellIdsToMerge[0])
        cell0PointIds = vtk.vtkIdList()
        cell0PointIds.DeepCopy(cell0ToMerge.GetPointIds())
        numberOfCell0Points = cell0PointIds.GetNumberOfIds()
        cell1ToMerge = self.Network.GetCell(self.CellIdsToMerge[1])
        cell1PointIds = vtk.vtkIdList()
        cell1PointIds.DeepCopy(cell1ToMerge.GetPointIds())
        numberOfCell1Points = cell1PointIds.GetNumberOfIds()
        reverse = [False,False]
        if cell0PointIds.GetId(numberOfCell0Points-1) == cell1PointIds.GetId(0):
            reverse = [False,False]
        elif cell0PointIds.GetId(numberOfCell0Points-1) == cell1PointIds.GetId(numberOfCell1Points-1):
            reverse = [False,True]
        elif cell0PointIds.GetId(0) == cell1PointIds.GetId(numberOfCell1Points-1):
            reverse = [True,True]
        elif cell0PointIds.GetId(0) == cell1PointIds.GetId(0):
            reverse = [True,False]
        else:
            self.PrintLog('Error: trying to merge non-adjacent segments.')
            return
        mergedLabel = self.NetworkLabelsArray.GetValue(self.CellIdsToMerge[0])
        self.RemoveCells(self.CellIdsToMerge)
        networkCellArray = self.Network.GetLines()
        cellId = networkCellArray.InsertNextCell(numberOfCell0Points+numberOfCell1Points-1)
        for i in range(numberOfCell0Points):
            loc = i
            if reverse[0]:
                loc = numberOfCell0Points - 1 - i
            networkCellArray.InsertCellPoint(cell0PointIds.GetId(loc))
        for i in range(1,numberOfCell1Points):
            loc = i
            if reverse[1]:
                loc = numberOfCell1Points - 1 - i
            networkCellArray.InsertCellPoint(cell1PointIds.GetId(loc))
        self.NetworkLabelsArray.InsertValue(cellId,mergedLabel)
        self.Network.SetLines(networkCellArray)
        self.Network.BuildCells()
        self.Network.Modified()
        self.NetworkTube.Modified()
        self.UpdateLabels()
        self.InitializeActiveSegment()
        self.CellIdsToMerge = []

    def RemoveCell(self,cellId):
        self.RemoveCells([cellId])

    def RemoveCells(self,cellIds):
        networkCellArray = vtk.vtkCellArray()
        networkLabelsArray = vtk.vtkStringArray()
        numberOfCells = self.Network.GetNumberOfCells()
        for i in range(numberOfCells):
            if i in cellIds:
                continue
            networkCellArray.InsertNextCell(self.Network.GetCell(i))
            networkLabelsArray.InsertNextValue(self.NetworkLabelsArray.GetValue(i))
        self.Network.SetLines(networkCellArray)
        self.NetworkLabelsArray.DeepCopy(networkLabelsArray)
        self.Network.BuildCells()
        self.Network.Modified()

    def SplitCell(self,cellId,subId,pcoord):
        splitPointId, hasSplit = self.SplitCellNoRemove(cellId,subId,pcoord)
        if hasSplit:
            self.RemoveCell(cellId)
        return splitPointId, hasSplit

    def SplitCellNoRemove(self,cellId,subId,pcoord):
        splitId = subId
        if pcoord > 0.5:
            splitId = subId+1
        cell = self.Network.GetCell(cellId)
        label = self.NetworkLabelsArray.GetValue(cellId)
        numberOfCellPoints = cell.GetNumberOfPoints()
        cellPointIds = cell.GetPointIds()
        splitPointId = cellPointIds.GetId(splitId)
        if splitId == 0 or splitId == numberOfCellPoints-1:
            return splitPointId, False
        lines = self.Network.GetLines()
        newCellId1 = lines.InsertNextCell(splitId+1)
        for i in range(splitId+1):
            lines.InsertCellPoint(cellPointIds.GetId(i))
        newCellId2 = lines.InsertNextCell(numberOfCellPoints-splitId)
        for i in range(splitId,numberOfCellPoints):
            lines.InsertCellPoint(cellPointIds.GetId(i))
        newLabel1 = ''
        newLabel2 = ''
        if label:
            newLabel1 = label + "1"
            newLabel2 = label + "2"
        self.NetworkLabelsArray.InsertValue(newCellId1,newLabel1)
        self.NetworkLabelsArray.InsertValue(newCellId2,newLabel2)
        self.Network.BuildCells()
        return splitPointId, True

    def LeftButtonPressCallback(self,obj,event):
        if self.PickMode != 'network':
            return
        if self.vmtkRenderer.RenderWindowInteractor.GetControlKey() == 0:
            return
        if self.Network.GetNumberOfCells() == 0:
            return
        cellId = self.SelectedCellId
        if cellId == -1:
            return
        subId = self.SelectedSubId
        pcoords = self.SelectedPCoords
        cell = self.Network.GetCell(cellId)
        if pcoords < 0.5:
            pointId = cell.GetPointIds().GetId(subId)
        else:
            pointId = cell.GetPointIds().GetId(subId+1)
        point = self.Network.GetPoint(pointId)
        radius = self.NetworkRadiusArray.GetValue(pointId)
        if self.OperationMode == 'add':
            if radius == 0.0:
                radius = 1.0
            if self.ActiveSegmentSeeds.GetNumberOfPoints() == 0:
                self.ActiveSegmentSeedsPoints.InsertNextPoint(point)
                self.ActiveSegmentSeedsRadiusArray.InsertNextValue(radius)
                self.ActiveSegmentSeeds.Modified()
                self.AttachedCellIds[0] = cellId
                self.AttachedSubIds[0] = subId
                self.AttachedPCoords[0] = pcoords
                self.Render()
                return
            if self.AttachedCellIds[1] != -1:
                return
            self.AttachedCellIds[1] = cellId
            self.AttachedSubIds[1] = subId
            self.AttachedPCoords[1] = pcoords
            if self.ActiveSegmentPoints.GetNumberOfPoints() == 0:
                self.ActiveSegmentPoints.InsertNextPoint(self.ActiveSegmentSeedsPoints.GetPoint(0))
                self.ActiveSegmentRadiusArray.InsertNextValue(self.ActiveSegmentSeedsRadiusArray.GetValue(0))
            self.ActiveSegmentSeedsPoints.InsertNextPoint(point)
            self.ActiveSegmentSeedsRadiusArray.InsertNextValue(radius)
            self.ActiveSegmentSeeds.Modified()
            self.ActiveSegmentPoints.InsertNextPoint(point)
            self.ActiveSegmentRadiusArray.InsertNextValue(radius)
            self.ActiveSegmentCellArray.Initialize()
            numberOfPoints = self.ActiveSegmentPoints.GetNumberOfPoints()
            self.ActiveSegmentCellArray.InsertNextCell(numberOfPoints)
            for i in range(numberOfPoints):
                self.ActiveSegmentCellArray.InsertCellPoint(i)
            self.ActiveSegment.Modified()
            self.Render()
        elif self.OperationMode == 'delete':
            self.InitializeSelection()
            self.RemoveCell(cellId)
            self.NetworkTube.Modified()
            self.UpdateLabels()
        elif self.OperationMode == 'split':
            numberOfCellPoints = cell.GetNumberOfPoints()
            self.SplitCell(cellId,subId,pcoords)
            self.NetworkTube.Modified()
            self.UpdateLabels()
        elif self.OperationMode == 'label':
            self.LabelCellId = cellId
            label = self.NetworkLabelsArray.GetValue(cellId)
            if label:
                self.PrintLog("Current label: %s" % label)
            self.vmtkRenderer.PromptAsync("Please input new label: ",self.QueryLabelCallback)
        elif self.OperationMode == 'merge':
            if self.ActiveSegment.GetNumberOfCells() > 1:
                self.InitializeActiveSegment()
                self.CellIdsToMerge = []
            self.CellIdsToMerge.append(cellId)
            numberOfCellPoints = cell.GetNumberOfPoints()
            activePointIds = []
            for i in range(numberOfCellPoints):
                pointId = cell.GetPointId(i)
                point = self.Network.GetPoint(pointId)
                radius = self.NetworkRadiusArray.GetTuple1(pointId)
                activePointId = self.ActiveSegmentPoints.InsertNextPoint(point)
                self.ActiveSegmentRadiusArray.InsertNextValue(radius)
                activePointIds.append(activePointId)
            self.ActiveSegmentCellArray.InsertNextCell(numberOfCellPoints)
            for i in range(numberOfCellPoints):
                self.ActiveSegmentCellArray.InsertCellPoint(activePointIds[i])
            self.ActiveSegment.Modified()
            self.NetworkTube.Modified()
            self.Render()

    def QueryLabelCallback(self,newLabel):
        self.NetworkLabelsArray.SetValue(self.LabelCellId,newLabel)
        self.LabelCellId = None
        self.InitializeSelection()
        self.Network.Modified()
        self.UpdateLabels()

    def InitializeSelection(self):
        self.SelectionPoints.SetNumberOfPoints(0)
        self.SelectionRadiusArray.SetNumberOfValues(0)
        self.SelectedCellId = -1
        self.SelectedSubId = -1
        self.SelectedPCoords = 0.0
        self.Selection.Modified()
        self.Render()

    def InitializeActiveSegment(self):
        self.ActiveSegmentSeedsPoints.SetNumberOfPoints(0)
        self.ActiveSegmentSeedsRadiusArray.SetNumberOfValues(0)
        self.ActiveSegmentSeeds.Modified()
        self.ActiveSegmentPoints.SetNumberOfPoints(0)
        self.ActiveSegmentCellArray.Initialize()
        self.ActiveSegmentCellArray.SetNumberOfCells(0)
        self.ActiveSegmentRadiusArray.SetNumberOfValues(0)
        self.ActiveSegment.Modified()
        self.AttachedCellIds = [-1,-1]
        self.AttachedSubIds = [-1,-1]
        self.AttachedPCoords = [0.0,0.0]
        self.Render()

    def MouseMoveCallback(self,obj,event):
        if self.PickMode != 'network':
            return
        if self.vmtkRenderer.RenderWindowInteractor.GetControlKey() == 0:
            if self.SelectedCellId != -1:
                self.InitializeSelection()
            return
        if self.Network.GetNumberOfCells() == 0:
            return
        eventPosition = self.vmtkRenderer.RenderWindowInteractor.GetEventPosition()
        result = self.CellPicker.Pick(float(eventPosition[0]),float(eventPosition[1]),0.0,self.vmtkRenderer.Renderer)
        if result == 0:
            return
        cellId = self.CellPicker.GetCellId()
        cell = self.Network.GetCell(cellId)
        subId = self.CellPicker.GetSubId()
        pcoords = self.CellPicker.GetPCoords()[0]
        if pcoords < 0.5:
            pointId = cell.GetPointIds().GetId(subId)
        else:
            pointId = cell.GetPointIds().GetId(subId+1)
        if self.vmtkRenderer.RenderWindowInteractor.GetShiftKey() == 1:
            if subId < cell.GetNumberOfPoints()/2:
                pointId = cell.GetPointIds().GetId(0)
            else:
                pointId = cell.GetPointIds().GetId(cell.GetNumberOfPoints()-1)
        point = self.Network.GetPoint(pointId)
        radius = self.NetworkRadiusArray.GetValue(pointId)
        if radius == 0.0:
            radius = 1.0
        self.SelectionPoints.InsertPoint(0,point)
        self.SelectionRadiusArray.InsertValue(0,radius)
        self.SelectedCellId = cellId
        self.SelectedSubId = subId
        self.SelectedPCoords = pcoords
        self.Selection.Modified()
        self.Render()

    def PlaneStartInteractionCallback(self,obj,event):
        if self.PickMode != 'image':
            return
        if self.OperationMode != 'add':
            return
        if self.vmtkRenderer.RenderWindowInteractor.GetControlKey() == 0:
            return
        point = obj.GetCurrentCursorPosition()
        radius = self.CurrentRadius
        if self.ActiveSegmentSeeds.GetNumberOfPoints() == 0:
            self.ActiveSegmentSeedsPoints.InsertNextPoint(point)
            self.ActiveSegmentSeedsRadiusArray.InsertNextValue(radius)
            self.ActiveSegmentSeeds.Modified()
            self.Render()
            return
        if self.AttachedCellIds[1] != -1:
            return
        if self.ActiveSegmentPoints.GetNumberOfPoints() == 0:
            self.ActiveSegmentPoints.InsertNextPoint(self.ActiveSegmentSeedsPoints.GetPoint(0))
            self.ActiveSegmentRadiusArray.InsertNextValue(self.ActiveSegmentSeedsRadiusArray.GetValue(0))
        self.ActiveSegmentSeedsPoints.InsertNextPoint(point)
        self.ActiveSegmentSeedsRadiusArray.InsertNextValue(radius)
        self.ActiveSegmentSeeds.Modified()
        self.ActiveSegmentPoints.InsertNextPoint(point)
        self.ActiveSegmentRadiusArray.InsertNextValue(radius)
        self.ActiveSegmentCellArray.Initialize()
        numberOfPoints = self.ActiveSegmentPoints.GetNumberOfPoints()
        self.ActiveSegmentCellArray.InsertNextCell(numberOfPoints)
        for i in range(numberOfPoints):
            self.ActiveSegmentCellArray.InsertCellPoint(i)
        self.ActiveSegment.Modified()
        self.Render()

    def FirstRender(self):
        self.vmtkRenderer.Render()

    def Render(self):
        self.vmtkRenderer.RenderWindow.Render()

    def UpdateAndRender(self):
        self.vmtkRenderer.Render(interactive=0)

    def RunActiveTube(self,segment):
        activeTubes = vmtkactivetubes.vmtkActiveTubes()
        activeTubes.Centerline = segment
        activeTubes.Image = self.FeatureImage
        activeTubes.NumberOfIterations = self.NumberOfIterations
        activeTubes.PotentialWeight = self.PotentialWeight
        activeTubes.StiffnessWeight = self.StiffnessWeight
        activeTubes.RadiusArrayName = self.RadiusArrayName
        activeTubes.Execute()
        segment.DeepCopy(activeTubes.Centerline)

    def Execute(self):

        if not self.Network:
            self.Network = vtk.vtkPolyData()
            networkPoints = vtk.vtkPoints()
            networkLines = vtk.vtkCellArray()
            radiusArray = vtk.vtkDoubleArray()
            radiusArray.SetName(self.RadiusArrayName)
            self.Network.SetPoints(networkPoints)
            self.Network.SetLines(networkLines)
            self.Network.GetPointData().AddArray(radiusArray)

        if not self.vmtkRenderer:
            self.vmtkRenderer = vmtkrenderer.vmtkRenderer()
            self.vmtkRenderer.Initialize()
            self.OwnRenderer = 1

        self.vmtkRenderer.ExitAfterTextInputMode = False
        self.vmtkRenderer.RegisterScript(self)

        if self.Image and (not self.PlaneWidgetX or not self.PlaneWidgetY or not self.PlaneWidgetZ):
            imageViewer = vmtkimageviewer.vmtkImageViewer()
            imageViewer.Image = self.Image
            imageViewer.vmtkRenderer = self.vmtkRenderer
            imageViewer.Display = 0
            imageViewer.Execute()
            self.PlaneWidgetX = imageViewer.PlaneWidgetX
            self.PlaneWidgetY = imageViewer.PlaneWidgetY
            self.PlaneWidgetZ = imageViewer.PlaneWidgetZ

        if self.Image:
            spacing = self.Image.GetSpacing()
            self.CurrentRadius = min(spacing)

        if self.UseActiveTubes and not self.FeatureImage:
            imageFeatures = vmtkimagefeatures.vmtkImageFeatures()
            imageFeatures.Image = self.Image
            imageFeatures.FeatureImageType = 'vtkgradient'
            imageFeatures.Execute()
            self.FeatureImage = imageFeatures.FeatureImage

        self.NetworkRadiusArray = self.Network.GetPointData().GetArray(self.RadiusArrayName)

        self.Network.GetPointData().SetActiveScalars(self.RadiusArrayName)

        networkMapper = vtk.vtkPolyDataMapper()
        networkMapper.SetInputData(self.Network)
        networkMapper.SetScalarModeToUseCellData()

        self.NetworkActor = vtk.vtkActor()
        self.NetworkActor.SetMapper(networkMapper)
        self.vmtkRenderer.Renderer.AddActor(self.NetworkActor)

        self.NetworkTube = vtk.vtkTubeFilter()
        self.NetworkTube.SetInputData(self.Network)
        self.NetworkTube.SetVaryRadiusToVaryRadiusByAbsoluteScalar()
        self.NetworkTube.SetNumberOfSides(20)
        networkTubeMapper = vtk.vtkPolyDataMapper()
        networkTubeMapper.SetInputConnection(self.NetworkTube.GetOutputPort())
        networkTubeMapper.ScalarVisibilityOff()
        networkTubeActor = vtk.vtkActor()
        networkTubeActor.SetMapper(networkTubeMapper)
        networkTubeActor.PickableOff()
        networkTubeActor.GetProperty().SetOpacity(0.2)
        self.vmtkRenderer.Renderer.AddActor(networkTubeActor)

        self.Selection = vtk.vtkPolyData()
        self.SelectionPoints = vtk.vtkPoints()
        self.SelectionRadiusArray = vtk.vtkDoubleArray()
        self.SelectionRadiusArray.SetName(self.RadiusArrayName)
        self.Selection.SetPoints(self.SelectionPoints)
        self.Selection.GetPointData().AddArray(self.SelectionRadiusArray)
        self.Selection.GetPointData().SetActiveScalars(self.RadiusArrayName)

        glyphs = vtk.vtkGlyph3D()
        glyphSource = vtk.vtkSphereSource()
        glyphSource.SetRadius(1.0)
        glyphSource.SetThetaResolution(20)
        glyphSource.SetPhiResolution(20)
        glyphs.SetInputData(self.Selection)
        glyphs.SetSourceConnection(glyphSource.GetOutputPort())
        glyphs.SetScaleModeToScaleByScalar()
        glyphs.SetScaleFactor(1.0)

        selectionMapper = vtk.vtkPolyDataMapper()
        selectionMapper.SetInputConnection(glyphs.GetOutputPort())

        self.SelectionActor = vtk.vtkActor()
        self.SelectionActor.SetMapper(selectionMapper)
        self.SelectionActor.GetProperty().SetColor(1.0,0.0,0.0)
        self.SelectionActor.GetProperty().SetOpacity(0.5)
        self.SelectionActor.PickableOff()
        self.vmtkRenderer.Renderer.AddActor(self.SelectionActor)

        self.ActiveSegmentSeeds = vtk.vtkPolyData()
        self.ActiveSegmentSeedsPoints = vtk.vtkPoints()
        self.ActiveSegmentSeedsRadiusArray = vtk.vtkDoubleArray()
        self.ActiveSegmentSeedsRadiusArray.SetName(self.RadiusArrayName)
        self.ActiveSegmentSeeds.SetPoints(self.ActiveSegmentSeedsPoints)
        self.ActiveSegmentSeeds.GetPointData().AddArray(self.ActiveSegmentSeedsRadiusArray)
        self.ActiveSegmentSeeds.GetPointData().SetActiveScalars(self.RadiusArrayName)

        activeSegmentSeedsGlyphs = vtk.vtkGlyph3D()
        activeSegmentSeedsGlyphSource = vtk.vtkSphereSource()
        activeSegmentSeedsGlyphSource.SetRadius(1.0)
        activeSegmentSeedsGlyphSource.SetThetaResolution(20)
        activeSegmentSeedsGlyphSource.SetPhiResolution(20)
        activeSegmentSeedsGlyphs.SetInputData(self.ActiveSegmentSeeds)
        activeSegmentSeedsGlyphs.SetSourceConnection(activeSegmentSeedsGlyphSource.GetOutputPort())
        activeSegmentSeedsGlyphs.SetScaleModeToScaleByScalar()
        activeSegmentSeedsGlyphs.SetScaleFactor(1.0)

        activeSegmentSeedsMapper = vtk.vtkPolyDataMapper()
        activeSegmentSeedsMapper.SetInputConnection(activeSegmentSeedsGlyphs.GetOutputPort())
        activeSegmentSeedsMapper.ScalarVisibilityOff()

        self.ActiveSegmentSeedsActor = vtk.vtkActor()
        self.ActiveSegmentSeedsActor.SetMapper(activeSegmentSeedsMapper)
        self.ActiveSegmentSeedsActor.GetProperty().SetColor(1.0,0.0,0.0)
        self.ActiveSegmentSeedsActor.GetProperty().SetOpacity(0.5)
        self.ActiveSegmentSeedsActor.PickableOff()
        self.vmtkRenderer.Renderer.AddActor(self.ActiveSegmentSeedsActor)

        self.ActiveSegment = vtk.vtkPolyData()
        self.ActiveSegmentPoints = vtk.vtkPoints()
        self.ActiveSegmentCellArray = vtk.vtkCellArray()
        self.ActiveSegmentRadiusArray = vtk.vtkDoubleArray()
        self.ActiveSegmentRadiusArray.SetName(self.RadiusArrayName)
        self.ActiveSegment.SetPoints(self.ActiveSegmentPoints)
        self.ActiveSegment.SetLines(self.ActiveSegmentCellArray)
        self.ActiveSegment.GetPointData().AddArray(self.ActiveSegmentRadiusArray)
        self.ActiveSegment.GetPointData().SetActiveScalars(self.RadiusArrayName)

        activeSegmentMapper = vtk.vtkPolyDataMapper()
        activeSegmentMapper.ScalarVisibilityOff()
        if self.SplineInterpolation and self.Image != None:
            splineFilter = vtk.vtkSplineFilter()
            splineFilter.SetInputData(self.ActiveSegment)
            splineFilter.SetSubdivideToLength()
            splineFilter.SetLength(2.0*min(self.Image.GetSpacing()))
            activeSegmentMapper.SetInputConnection(splineFilter.GetOutputPort())
        else:
            activeSegmentMapper.SetInputData(self.ActiveSegment)

        self.ActiveSegmentActor = vtk.vtkActor()
        self.ActiveSegmentActor.SetMapper(activeSegmentMapper)
        self.ActiveSegmentActor.GetProperty().SetColor(1.0,1.0,1.0)
        self.ActiveSegmentActor.GetProperty().SetLineWidth(3.0)
        self.ActiveSegmentActor.PickableOff()
        self.vmtkRenderer.Renderer.AddActor(self.ActiveSegmentActor)

        activeTube = vtk.vtkTubeFilter()
        activeTube.SetInputData(activeSegmentMapper.GetInput())
        activeTube.SetVaryRadiusToVaryRadiusByAbsoluteScalar()
        activeTube.SetNumberOfSides(20)
        activeTubeMapper = vtk.vtkPolyDataMapper()
        activeTubeMapper.SetInputConnection(activeTube.GetOutputPort())
        activeTubeMapper.ScalarVisibilityOff()
        activeTubeActor = vtk.vtkActor()
        activeTubeActor.SetMapper(activeTubeMapper)
        activeTubeActor.PickableOff()
        activeTubeActor.GetProperty().SetOpacity(0.6)
        self.vmtkRenderer.Renderer.AddActor(activeTubeActor)

        self.NetworkLabelsArray = vtk.vtkStringArray.SafeDownCast(self.Network.GetCellData().GetAbstractArray(self.LabelsArrayName))
        if not self.NetworkLabelsArray:
            self.NetworkLabelsArray = vtk.vtkStringArray()
            self.NetworkLabelsArray.SetName(self.LabelsArrayName)
            self.NetworkLabelsArray.SetNumberOfValues(self.Network.GetNumberOfCells())
            for i in range(self.Network.GetNumberOfCells()):
                self.NetworkLabelsArray.SetValue(i,'')
            self.Network.GetCellData().AddArray(self.NetworkLabelsArray)
        self.CellCenters = vtk.vtkCellCenters()
        self.CellCenters.SetInputData(self.Network)
        self.CellCenters.VertexCellsOff()
        self.CellCenters.Update()

        labeledMapper = vtk.vtkLabeledDataMapper()
        labeledMapper.SetInputConnection(self.CellCenters.GetOutputPort())
        labeledMapper.SetLabelModeToLabelFieldData()
        labeledMapper.SetFieldDataName(self.LabelsArrayName)
        labeledMapper.GetLabelTextProperty().SetFontFamilyToArial()
        labeledMapper.GetLabelTextProperty().BoldOff()
        labeledMapper.GetLabelTextProperty().ItalicOff()
        labeledMapper.GetLabelTextProperty().ShadowOff()

        self.LabelsActor = vtk.vtkActor2D()
        self.LabelsActor.SetMapper(labeledMapper)
        self.LabelsActor.VisibilityOff()
        self.vmtkRenderer.Renderer.AddActor(self.LabelsActor)

        self.CellPicker = vtk.vtkCellPicker()
        self.CellPicker.SetTolerance(1E-2)
        self.CellPicker.InitializePickList()
        self.CellPicker.AddPickList(self.NetworkActor)
        self.CellPicker.PickFromListOn()

        self.vmtkRenderer.AddKeyBinding('a','Add mode.',self.AddCallback)
        self.vmtkRenderer.AddKeyBinding('d','Delete mode.',self.DeleteCallback)
        self.vmtkRenderer.AddKeyBinding('m','Merge mode.',self.MergeCallback)
        self.vmtkRenderer.AddKeyBinding('s','Split mode.',self.SplitCallback)
        self.vmtkRenderer.AddKeyBinding('l','Label mode.',self.LabelCallback)
        self.vmtkRenderer.AddKeyBinding('Tab','Show labels.',self.ShowLabelCallback)
        self.vmtkRenderer.RenderWindowInteractor.AddObserver("KeyReleaseEvent", self.KeyReleaseCallback)
        self.vmtkRenderer.RenderWindowInteractor.AddObserver("LeftButtonPressEvent", self.LeftButtonPressCallback)
        self.vmtkRenderer.RenderWindowInteractor.AddObserver("MouseMoveEvent", self.MouseMoveCallback)

        if self.PlaneWidgetX:
            self.PlaneWidgetX.UseContinuousCursorOn()
            self.PlaneWidgetX.AddObserver("StartInteractionEvent", self.PlaneStartInteractionCallback)
        if self.PlaneWidgetY:
            self.PlaneWidgetY.UseContinuousCursorOn()
            self.PlaneWidgetY.AddObserver("StartInteractionEvent", self.PlaneStartInteractionCallback)
        if self.PlaneWidgetZ:
            self.PlaneWidgetZ.UseContinuousCursorOn()
            self.PlaneWidgetZ.AddObserver("StartInteractionEvent", self.PlaneStartInteractionCallback)

        self.FirstRender()

        self.Surface = self.NetworkTube.GetOutput()

        if self.OwnRenderer:
            self.vmtkRenderer.Deallocate()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkimagereslice.py,v $
## Language:  Python
## Date:      $Date: 2006/07/17 09:53:14 $
## Version:   $Revision: 1.8 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this class was improved by
##       Hugo Gratama van Andel
##       Academic Medical Centre - University of Amsterdam
##       Dept. Biomedical Engineering  & Physics

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import pypes


class vmtkImageReslice(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Image = None
        self.ReferenceImage = None

        self.OutputSpacing = []
        self.OutputOrigin = []
        self.OutputExtent = []

        self.Interpolation = 'linear'
        self.Cast = 1

        self.BackgroundLevel = 0.0

        self.MatrixCoefficients = []
        self.InvertMatrix = 0
        self.Matrix4x4 = None
        self.Rotation = [0.0,0.0,0.0]
        self.Translation = [0.0,0.0,0.0]
        self.Scaling = [1.0,1.0,1.0]

        self.TransformInputSampling = 1

        self.SetScriptName('vmtkimagereslice')
        self.SetScriptDoc('reslice an image based on user-specified parameters or on a reference image')
        self.SetInputMembers([
            ['Image','i','vtkImageData',1,'','the input image','vmtkimagereader'],
            ['ReferenceImage','r','vtkImageData',1,'','the reference image','vmtkimagereader'],
            ['OutputSpacing','spacing','float',3,'','the output spacing'],
            ['OutputOrigin','origin','float',3,'','the output origin'],
            ['OutputExtent','extent','int',6,'','the output extent'],
            ['Interpolation','interpolation','str',1,'["nearestneighbor","linear","cubic"]','interpolation during reslice'],
            ['Cast','cast','bool',1,'','toggle cast image to float type'],
            ['BackgroundLevel','background','float',1,'','the output image background'],
            ['Matrix4x4','matrix4x4','vtkMatrix4x4',1,'','the input transform matrix'],
            ['MatrixCoefficients','matrix','float',16,'','coefficients of transform matrix'],
            ['InvertMatrix','invert','bool',1,'','invert matrix before applying transformation'],
            ['Rotation','rotation','float',3,'','rotations around the x-,y- and z-axis'],
            ['Translation','translation','float',3,'','translation in the x-,y- and z-directions'],
            ['Scaling','scaling','float',3,'','scaling of the x-,y- and z-directions'],
            ['TransformInputSampling','transforminputsampling','bool',1,'','transform spacing, origin and extent of the Input (or the InformationInput) according to the direction cosines and origin of the ResliceAxes before applying them as the default output spacing, origin and extent']
            ])
        self.SetOutputMembers([
            ['Image','o','vtkImageData',1,'','the output image','vmtkimagewriter']
            ])

    def Execute(self):

        if self.Image == None:
            self.PrintError('Error: No input image.')

        if self.Cast:
            cast = vtk.vtkImageCast()
            cast.SetInputData(self.Image)
            cast.SetOutputScalarTypeToFloat()
            cast.Update()
            self.Image = cast.GetOutput()

        resliceFilter = vtk.vtkImageReslice()
        resliceFilter.SetInputData(self.Image)
        if self.ReferenceImage:
            resliceFilter.SetInformationInput(self.ReferenceImage)
        else:
            if self.OutputSpacing:
                resliceFilter.SetOutputSpacing(self.OutputSpacing)
            if self.OutputOrigin:
                resliceFilter.SetOutputOrigin(self.OutputOrigin)
            if self.OutputExtent:
                resliceFilter.SetOutputExtent(self.OutputExtent)
        if self.Interpolation == 'nearestneighbor':
            resliceFilter.SetInterpolationModeToNearestNeighbor()
        elif self.Interpolation == 'linear':
            resliceFilter.SetInterpolationModeToLinear()
        elif self.Interpolation == 'cubic':
            resliceFilter.SetInterpolationModeToCubic()
        else:
            self.PrintError('Error: unsupported interpolation mode')
        resliceFilter.SetBackgroundLevel(self.BackgroundLevel)

        if self.TransformInputSampling:
            resliceFilter.TransformInputSamplingOn()
        else:
            resliceFilter.TransformInputSamplingOff()

        if not self.Matrix4x4:
            if self.MatrixCoefficients != []:
                self.PrintLog('Setting up transform matrix using specified coefficients')
                self.Matrix4x4 = vtk.vtkMatrix4x4()
                self.Matrix4x4.DeepCopy(self.MatrixCoefficients)
            elif self.Translation != [0.0,0.0,0.0] or self.Rotation != [0.0,0.0,0.0] or self.Scaling != [1.0,1.0,1.0]:
                self.PrintLog('Setting up transform matrix using specified translation, rotation and/or scaling')
                transform = vtk.vtkTransform()
                transform.RotateX(self.Rotation[0])
                transform.RotateY(self.Rotation[1])
                transform.RotateZ(self.Rotation[2])
                transform.Translate(self.Translation[0], self.Translation[1], self.Translation[2])
                transform.Scale(self.Scaling[0], self.Scaling[1], self.Scaling[2])
                self.Matrix4x4 = vtk.vtkMatrix4x4()
                self.Matrix4x4.DeepCopy(transform.GetMatrix())

        if self.InvertMatrix and self.Matrix4x4:
            self.Matrix4x4.Invert()

        if self.Matrix4x4:
            transform = vtk.vtkMatrixToLinearTransform()
            transform.SetInput(self.Matrix4x4)
            resliceFilter.SetResliceTransform(transform)

        resliceFilter.Update()

        self.Image = resliceFilter.GetOutput()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program: VMTK
## Language:  Python
## Date:      January 12, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtksurfaceappend as surfaceappend


def test_append_defaults(aorta_surface, aorta_surface2, compare_surfaces):
    name = __name__ + '_test_append_defaults.vtp'
    appender = surfaceappend.vmtkSurfaceAppend()
    appender.Surface = aorta_surface
    appender.Surface2 = aorta_surface2
    appender.Execute()

    assert compare_surfaces(appender.Surface, name)



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkcenterlinesmoothing.py,v $
## Language:  Python
## Date:      $Date: 2006/07/17 09:52:56 $
## Version:   $Revision: 1.1 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
from vmtk import vtkvmtk
import sys

from vmtk import pypes


class vmtkCenterlineSmoothing(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Centerlines = None

        self.NumberOfSmoothingIterations = 100
        self.SmoothingFactor = 0.1

        self.SetScriptName('vmtkcenterlinesmoothing')
        self.SetScriptDoc('smooth centerlines with a moving average filter')
        self.SetInputMembers([
            ['Centerlines','i','vtkPolyData',1,'','the input centerlines','vmtksurfacereader'],
            ['NumberOfSmoothingIterations','iterations','int',1,'(0,)'],
            ['SmoothingFactor','factor','float',1,'(0.0,)']
            ])
        self.SetOutputMembers([
            ['Centerlines','o','vtkPolyData',1,'','the output centerlines','vmtksurfacewriter']
            ])

    def Execute(self):

        if self.Centerlines == None:
            self.PrintError('Error: No input centerlines.')

        centerlineSmoothing = vtkvmtk.vtkvmtkCenterlineSmoothing()
        centerlineSmoothing.SetInputData(self.Centerlines)
        centerlineSmoothing.SetNumberOfSmoothingIterations(self.NumberOfSmoothingIterations)
        centerlineSmoothing.SetSmoothingFactor(self.SmoothingFactor)
        centerlineSmoothing.Update()

        self.Centerlines = centerlineSmoothing.GetOutput()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtksurfacereferencesystemtransform.py,v $
## Language:  Python
## Date:      $Date: 2005/09/14 09:49:59 $
## Version:   $Revision: 1.7 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import math
import sys

from vmtk import pypes


class vmtkSurfaceReferenceSystemTransform(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.ReferenceSystems = None

        self.ReferenceSystemId = -1

        self.ReferenceSystemsIdArrayName = ''
        self.ReferenceSystemsNormal1ArrayName = ''
        self.ReferenceSystemsNormal2ArrayName = ''

        self.ReferenceOrigin = [0.0, 0.0, 0.0]
        self.ReferenceNormal1 = [0.0, 0.0, 1.0]
        self.ReferenceNormal2 = [0.0, 1.0, 0.0]

        self.SetScriptName('vmtksurfacereferencesystemtransform')
        self.SetScriptDoc('translate and rotate a surface in order to orient its reference system with a target reference system')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['ReferenceOrigin','referenceorigin','float',3,'','origin of the target reference system'],
            ['ReferenceNormal1','referencenormal1','float',3,'','first normal of the target reference system'],
            ['ReferenceNormal2','referencenormal2','float',3,'','second normal of the target reference system'],
            ['ReferenceSystems','referencesystems','vtkPolyData',1,'','reference systems of the input surface represented by a set of points with two normals each','vmtksurfacereader'],
            ['ReferenceSystemId','referencesystemid','int',1,'','id of the reference system to use'],
            ['ReferenceSystemsIdArrayName','referencesystemidsarray','str',1,'','name of the array where reference system ids are stored'],
            ['ReferenceSystemsNormal1ArrayName','normal1array','str',1,'','name of the array where the first normals defining the reference systems are stored'],
            ['ReferenceSystemsNormal2ArrayName','normal2array','str',1,'','name of the array where the second normals defining the reference systems are stored']
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','the output surface','vmtksurfacewriter']
            ])

    def ComputeAngle(self,in_vector0,in_vector1):
        vector0 = [in_vector0[0], in_vector0[1], in_vector0[2]]
        vector1 = [in_vector1[0], in_vector1[1], in_vector1[2]]
        vtk.vtkMath.Normalize(vector0)
        vtk.vtkMath.Normalize(vector1)
        sum = [
            vector0[0] + vector1[0],
            vector0[1] + vector1[1],
            vector0[2] + vector1[2]]
        difference = [
            vector0[0] - vector1[0],
            vector0[1] - vector1[1],
            vector0[2] - vector1[2]]
        sumNorm = vtk.vtkMath.Norm(sum)
        differenceNorm = vtk.vtkMath.Norm(difference)
##         angle = 2.0 * math.atan2(differenceNorm,sumNorm) + math.pi
        angle = 2.0 * math.atan2(differenceNorm,sumNorm)
        return angle

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: No Surface.')

        if self.ReferenceSystems == None:
            self.PrintError('Error: No ReferenceSystems.')

        referenceSystemsNormal1Array = self.ReferenceSystems.GetPointData().GetArray(self.ReferenceSystemsNormal1ArrayName)
        referenceSystemsNormal2Array = self.ReferenceSystems.GetPointData().GetArray(self.ReferenceSystemsNormal2ArrayName)

        referenceSystemPointId = 0

        if self.ReferenceSystemId != -1:
            referenceSystemsIdArray = self.ReferenceSystems.GetPointData().GetArray(self.ReferenceSystemsIdArrayName)
            for i in range(self.ReferenceSystems.GetNumberOfPoints()):
                referenceSystemId = int(referenceSystemsIdArray.GetTuple1(i))
                if referenceSystemId == self.ReferenceSystemId:
                    referenceSystemPointId = i
                    break

        currentOrigin = self.ReferenceSystems.GetPoint(referenceSystemPointId)
        currentNormal1 = referenceSystemsNormal1Array.GetTuple3(referenceSystemPointId)
        currentNormal2 = referenceSystemsNormal2Array.GetTuple3(referenceSystemPointId)

        transform = vtk.vtkTransform()
        transform.PostMultiply()

        translation = [0.0, 0.0, 0.0]
        translation[0] = self.ReferenceOrigin[0] - currentOrigin[0]
        translation[1] = self.ReferenceOrigin[1] - currentOrigin[1]
        translation[2] = self.ReferenceOrigin[2] - currentOrigin[2]

        transform.Translate(translation)

        cross = [0.0,0.0,0.0]

        vtk.vtkMath.Cross(currentNormal1,self.ReferenceNormal1,cross)
        vtk.vtkMath.Normalize(cross)
        angle = self.ComputeAngle(currentNormal1,self.ReferenceNormal1)
        angle = angle / (2.0*vtk.vtkMath.Pi()) * 360.0;
        transform.RotateWXYZ(angle,cross)

        transformedNormal2 = transform.TransformNormal(currentNormal2)
        vtk.vtkMath.Cross(transformedNormal2,self.ReferenceNormal2,cross)
        vtk.vtkMath.Normalize(cross)
        angle = self.ComputeAngle(transformedNormal2,self.ReferenceNormal2)
        angle = angle / (2.0*vtk.vtkMath.Pi()) * 360.0;
        transform.RotateWXYZ(angle,cross)

        transformFilter = vtk.vtkTransformPolyDataFilter()
        transformFilter.SetInputData(self.Surface)
        transformFilter.SetTransform(transform)
        transformFilter.Update()

        self.Surface = transformFilter.GetOutput()


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program: VMTK
## Language:  Python
## Date:      January 10, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import os
from hashlib import sha1
import vmtk.vmtkimagetonumpy as wrap
import vmtk.vmtknumpytoimage as wrap2
import vmtk.vmtkimagecompare as comp
import vmtk.vmtkimagereader as r


def test_compare_same_image(aorta_image):

    comparer = comp.vmtkImageCompare()
    comparer.Method = 'subtraction'
    comparer.Image = aorta_image
    comparer.ReferenceImage = aorta_image
    comparer.Execute()

    assert comparer.Result is True


def test_compare_not_same_image(aorta_image):

    wraper = wrap.vmtkImageToNumpy()
    wraper.Image = aorta_image
    wraper.Execute()

    multiplied = wraper.ArrayDict
    multiplied['PointData']['ImageScalars'] = multiplied['PointData']['ImageScalars'] * 2

    back = wrap2.vmtkNumpyToImage()
    back.ArrayDict = multiplied
    back.Execute()

    comparer = comp.vmtkImageCompare()
    comparer.Method = 'subtraction'
    comparer.Image = aorta_image
    comparer.ReferenceImage = back.Image
    comparer.Execute()

    assert comparer.Result is False

#TODO: See why subtraction fails



#!${PYTHON_SHEBANG}

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import pypes

pypetest = 'pypeTest'


class pypeTest(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.TestName = None
        self.PypeTestLog = ''
        self.TestInput = None
        self.Condition = None
        self.ConditionValue = None
        self.ConditionType = None

        self.SetScriptName('pypetest')
        self.SetScriptDoc('tests a script property against a condition')
        self.SetInputMembers([
            ['TestName','name','str',1,'','name of the test'],
            ['TestInput','i','test',1,'','log'],
            ['Condition','condition','str',1,'["equalto","differentfrom","greaterthan","lessthan","nonnone"]','condition type'],
            ['ConditionValue','value','str',1,'','condition value'],
            ['ConditionType','type','str',1,'["str","int","float","bool"]','condition type']
            ])
        self.SetOutputMembers([
            ['PypeTestLog','log','str',1,'','log']
            ])

    def castValue(self,val):
        if not self.ConditionType:
            self.PrintError('Error: No condition type.')
        if self.ConditionType == 'str':
            return str(val)
        elif self.ConditionType == 'int':
            return int(val)
        elif self.ConditionType == 'float':
            return float(val)
        elif self.ConditionType == 'bool':
            if val == True or val in ['1','True','true']:
                return True
            else:
                return False

    def Execute(self):

        self.PrintLog('Testing')

        if not self.TestName:
            self.PrintError('Error: No test name.')
        if self.TestInput == None:
            self.PrintError('Error: No test input.')
        if not self.Condition:
            self.PrintError('Error: No condition.')

        passed = False
        self.CompareLog = 'failed'

        if self.Condition == 'equalto':
            if self.ConditionValue and self.castValue(self.TestInput) == self.castValue(self.ConditionValue):
                passed = True
        elif self.Condition == 'differentfrom':
            if self.ConditionValue and self.castValue(self.TestInput) != self.castValue(self.ConditionValue):
                passed = True
        elif self.Condition == 'greaterthan':
            if self.ConditionValue and self.castValue(self.TestInput) > self.castValue(self.ConditionValue):
                passed = True
        elif self.Condition == 'lessthan':
            if self.ConditionValue and self.castValue(self.TestInput) < self.castValue(self.ConditionValue):
                passed = True
        elif self.Condition == 'nonnone':
            if self.TestInput != None :
                passed = True

        if passed:
            self.CompareLog = 'passed'

        self.PypeTestLog = "%s.%s" % (self.TestName, self.CompareLog)


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program: VMTK
## Language:  Python
## Date:      January 10, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtksurfacesmoothing as smoothing


## Windowed sinc smoothing is a Chebyshev polynomial approximation whose
## degree equals the iteration count, so it needs a reasonable number of
## iterations to define a meaningful filter. The script's default single
## iteration is degenerate: vtkWindowedSincPolyDataFilter cannot build its
## kernel from it and its output in that regime is implementation-defined
## (it changed with the VTK 9.2 rewrite of the filter). The tests therefore
## pass explicit, meaningful parameters.

def test_taubin(aorta_surface, compare_surfaces):
    name = __name__ + '_test_taubin.vtp'
    smoother = smoothing.vmtkSurfaceSmoothing()
    smoother.Surface = aorta_surface
    smoother.Method = 'taubin'
    smoother.NumberOfIterations = 30
    smoother.PassBand = 0.1
    smoother.Execute()

    assert compare_surfaces(smoother.Surface, name) == True


def test_taubin_change_passband(aorta_surface, compare_surfaces):
    name = __name__ + '_test_taubin_change_passband.vtp'
    smoother = smoothing.vmtkSurfaceSmoothing()
    smoother.Surface = aorta_surface
    smoother.Method = 'taubin'
    smoother.NumberOfIterations = 30
    smoother.PassBand = 0.01
    smoother.Execute()

    assert compare_surfaces(smoother.Surface, name) == True


def test_laplace(aorta_surface, compare_surfaces):
    name = __name__ + '_test_laplace.vtp'
    smoother = smoothing.vmtkSurfaceSmoothing()
    smoother.Surface = aorta_surface
    smoother.Method = 'laplace'
    smoother.Execute()

    assert compare_surfaces(smoother.Surface, name) == True


def test_laplace_change_relaxation(aorta_surface, compare_surfaces):
    name = __name__ + '_test_laplace_change_relaxation.vtp'
    smoother = smoothing.vmtkSurfaceSmoothing()
    smoother.Surface = aorta_surface
    smoother.Method = 'laplace'
    smoother.RelaxationFactor = 0.05
    smoother.Execute()

    assert compare_surfaces(smoother.Surface, name) == True



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkmeshtosurface.py,v $
## Language:  Python
## Date:      $Date: 2006/04/06 16:47:46 $
## Version:   $Revision: 1.8 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import pypes


class vmtkMeshToSurface(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.Mesh = None
        self.CleanOutput = 0

        self.SetScriptName('vmtkmeshtosurface')
        self.SetScriptDoc('convert a mesh to a surface by throwing out volume elements and (optionally) the relative points')
        self.SetInputMembers([
            ['Mesh','i','vtkUnstructuredGrid',1,'','the input mesh','vmtkmeshreader'],
            ['CleanOutput','cleanoutput','bool',1,'','toggle cleaning the unused points']
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','the output surface','vmtksurfacewriter']])

    def Execute(self):

        if self.Mesh == None:
            self.PrintError('Error: No input mesh.')

        meshToSurfaceFilter = vtk.vtkGeometryFilter()
        meshToSurfaceFilter.SetInputData(self.Mesh)
        meshToSurfaceFilter.Update()

        self.Surface = meshToSurfaceFilter.GetOutput()

        if self.CleanOutput == 1:
            cleaner = vtk.vtkCleanPolyData()
            cleaner.SetInputConnection(meshToSurfaceFilter.GetOutputPort())
            cleaner.Update()
            self.Surface = cleaner.GetOutput()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program: VMTK
## Language:  Python
## Date:      May 2, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtkcenterlineimage as centerlineimage
import sys


@pytest.fixture(scope='module')
def per_system_compare_image_name():
    name = __name__ + '_test_default_parameters_'
    if sys.platform in ['win32', 'win64', 'cygwin']:
        name = name + 'windows.vti'
    elif sys.platform == 'darwin':
        name = name + 'mac.vti'
    else:
        name = name + 'linux.vti'
    return name


def test_default_parameters(aorta_surface, compare_images, per_system_compare_image_name):
    centImage = centerlineimage.vmtkCenterlineImage()
    centImage.Surface = aorta_surface
    centImage.Execute()

    assert compare_images(centImage.Image, per_system_compare_image_name) == True



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkbranchmapping.py,v $
## Language:  Python
## Date:      $Date: 2005/09/14 09:48:31 $
## Version:   $Revision: 1.11 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
from vmtk import vtkvmtk
import sys

from vmtk import pypes


class vmtkBranchMapping(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.Centerlines = None
        self.ReferenceSystems = None

        self.AbscissasArrayName = 'Abscissas'
        self.NormalsArrayName = 'ParallelTransportNormals'
        self.GroupIdsArrayName = 'GroupIds'
        self.CenterlineIdsArrayName = 'CenterlineIds'
        self.TractIdsArrayName = 'TractIds'
        self.ReferenceSystemsNormalArrayName = 'Normal'
        self.RadiusArrayName = 'MaximumInscribedSphereRadius'
        self.BlankingArrayName = 'Blanking'

        self.AngularMetricArrayName = 'AngularMetric'
        self.AbscissaMetricArrayName = 'AbscissaMetric'

        self.HarmonicMappingArrayName = 'HarmonicMapping'
        self.BoundaryMetricArrayName = 'BoundaryMetric'
        self.StretchedMappingArrayName = 'StretchedMapping'

        self.SetScriptName('vmtkbranchmapping')
        self.SetScriptDoc('constructs a harmonic function over each vascular segment; maps and stretches the longitudinal metric to correctly account for the presence of insertion regions at bifurcations; the additional StretchedMapping array is added to the surface')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','','vmtksurfacereader'],
            ['Centerlines','centerlines','vtkPolyData',1,'','','vmtksurfacereader'],
            ['ReferenceSystems','referencesystems','vtkPolyData',1,'','','vmtksurfacereader'],
            ['AbscissasArrayName','abscissasarray','str',1],
            ['NormalsArrayName','normalsarray','str',1],
            ['GroupIdsArrayName','groupidsarray','str',1],
            ['CenterlineIdsArrayName','centerlineidsarray','str',1],
            ['TractIdsArrayName','tractidsarray','str',1],
            ['ReferenceSystemsNormalArrayName','referencesystemsnormalarray','str',1],
            ['RadiusArrayName','radiusarray','str',1],
            ['BlankingArrayName','blankingarray','str',1],
            ['AngularMetricArrayName','angularmetricarray','str',1],
            ['HarmonicMappingArrayName','harmonicmappingarray','str',1],
            ['AbscissaMetricArrayName','abscissametricarray','str',1],
            ['BoundaryMetricArrayName','boundarymetricarray','str',1],
            ['StretchedMappingArrayName','stretchedmappingarray','str',1]
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','','vmtksurfacewriter'],
            ['HarmonicMappingArrayName','harmonicmappingarray','str',1],
            ['BoundaryMetricArrayName','boundarymetricarray','str',1],
            ['StretchedMappingArrayName','stretchedmappingarray','str',1]
            ])

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: No input surface.')

        if self.Centerlines == None:
            self.PrintError('Error: No input centerlines.')

        if self.ReferenceSystems == None:
            self.PrintError('Error: No input reference systems.')

        self.PrintLog('Computing boundary metric')
        boundaryMetricFilter = vtkvmtk.vtkvmtkPolyDataReferenceSystemBoundaryMetricFilter()
        boundaryMetricFilter.SetInputData(self.Surface)
        boundaryMetricFilter.SetBoundaryMetricArrayName(self.BoundaryMetricArrayName)
        boundaryMetricFilter.SetGroupIdsArrayName(self.GroupIdsArrayName)
        boundaryMetricFilter.SetCenterlines(self.Centerlines)
        boundaryMetricFilter.SetCenterlineAbscissasArrayName(self.AbscissasArrayName)
        boundaryMetricFilter.SetCenterlineRadiusArrayName(self.RadiusArrayName)
        boundaryMetricFilter.SetCenterlineGroupIdsArrayName(self.GroupIdsArrayName)
        boundaryMetricFilter.SetCenterlineTractIdsArrayName(self.TractIdsArrayName)
        boundaryMetricFilter.SetCenterlineIdsArrayName(self.CenterlineIdsArrayName)
        boundaryMetricFilter.SetReferenceSystems(self.ReferenceSystems)
        boundaryMetricFilter.SetReferenceSystemGroupIdsArrayName(self.GroupIdsArrayName)
        boundaryMetricFilter.Update()

        self.PrintLog('Computing harmonic mapping')
        harmonicMappingFilter = vtkvmtk.vtkvmtkPolyDataMultipleCylinderHarmonicMappingFilter()
        harmonicMappingFilter.SetInputConnection(boundaryMetricFilter.GetOutputPort())
        harmonicMappingFilter.SetHarmonicMappingArrayName(self.HarmonicMappingArrayName)
        harmonicMappingFilter.SetGroupIdsArrayName(self.GroupIdsArrayName)
        harmonicMappingFilter.Update()

        self.PrintLog('Stretching harmonic mapping')
        stretchFilter = vtkvmtk.vtkvmtkPolyDataStretchMappingFilter()
        stretchFilter.SetInputConnection(harmonicMappingFilter.GetOutputPort())
        stretchFilter.SetStretchedMappingArrayName(self.StretchedMappingArrayName)
        stretchFilter.SetHarmonicMappingArrayName(self.HarmonicMappingArrayName)
        stretchFilter.SetGroupIdsArrayName(self.GroupIdsArrayName)
        stretchFilter.SetMetricArrayName(self.AbscissaMetricArrayName)
        stretchFilter.SetBoundaryMetricArrayName(self.BoundaryMetricArrayName)
        stretchFilter.UseBoundaryMetricOn()
        stretchFilter.Update()

        self.Surface = stretchFilter.GetOutput()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Module:    $RCSfile: vmtknumpyreader.py,v $
## Language:  Python
## Date:      June 10, 2017
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this class was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo
##       The Jacobs Institute

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import pypes
import pickle
import os

try:
    import numpy as np
except ImportError:
    raise ImportError('Unable to Import vmtknumpyreader module, numpy is not installed')


class vmtkNumpyReader(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.InputFileName = ''
        self.ArrayDict = None
        self.Format = ''
        self.GuessFormat = 1

        self.SetScriptName('vmtkNumpyReader')
        self.SetScriptDoc('reads a pickled (or hdf5) file containing a nested dictionary of numpy arrays generated by'
                          'vmtkcenterlinestonumpy, vmtkimagetonumpy, or vmtksurfacetonumpy from disk and stores the'
                          'object in memory into a standard dictionary of numpy arrays.')

        self.SetInputMembers([
            ['Format','f','str',1,'["pickle","hdf5"]','file format'],
            ['GuessFormat','guessformat','bool', 1,'','guess file format from extension'],
            ['InputFileName','ifile','str',1,'','the input file name']])
        self.SetOutputMembers([
            ['ArrayDict','o','dict',1,'','the output dictionary']
        ])

    def ReadPickleFile(self):
        self.PrintLog('Reading Pickle File')
        with open(self.InputFileName, 'rb') as infile:
            self.ArrayDict = pickle.load(infile)

    def ReadHDF5File(self):
        """
        Load a dictionary from an HDF5 file.
        """

        try:
            import h5py
        except ImportError:
            self.PrintError('ImportError: Unable to Write to hdf5. h5py module not installed')
            raise ImportError('Unable to Write to hdf5. h5py module not installed')

        def recursively_load_dict_contents_from_group(h5file, path):
            """
            Load contents of an HDF5 group. If further groups are encountered,
            treat them like dicts and continue to load them recursively.
            """
            ans = {}
            for key, item in h5file[path].items():
                if isinstance(item, h5py._hl.dataset.Dataset):
                    ans[key] = item.value
                elif isinstance(item, h5py._hl.group.Group):
                    ans[key] = recursively_load_dict_contents_from_group(h5file, path + key + '/')
            return ans

        with h5py.File(self.InputFileName, 'r') as h5file:
            self.ArrayDict = recursively_load_dict_contents_from_group(h5file, '/')

    def Execute(self):
        extensionFormats = {'pickle':'pickle',
                            'hdf5':'hdf5'}

        if self.InputFileName == 'BROWSER':
            import tkinter.filedialog
            import os.path
            initialDir = pypes.pypeScript.lastVisitedPath
            self.InputFileName = tkinter.filedialog.askopenfilename(title="Input file",initialdir=initialDir)
            pypes.pypeScript.lastVisitedPath = os.path.dirname(self.InputFileName)
            if not self.InputFileName:
                self.PrintError('Error: no InputFileName.')

        if self.GuessFormat and self.InputFileName and not self.Format:
            import os.path
            extension = os.path.splitext(self.InputFileName)[1]
            if extension:
                extension = extension[1:]
                if extension in list(extensionFormats.keys()):
                    self.Format = extensionFormats[extension]

        if (self.InputFileName == ''):
            self.PrintError('Error: no InputFileName.')

        self.PrintLog('Reading File')
        if self.Format == 'pickle':
            self.ReadPickleFile()
        elif self.Format == 'hdf5':
            self.ReadHDF5File()
        else:
            self.PrintError('Error: unsupported format '+ self.Format + '.')

        self.Output = self.ArrayDict


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python
"""Smoke test for the vmtk binary wheel.

Run against an installed vmtk wheel (cibuildwheel runs it as the test
command for every built wheel). It verifies that:

  * the wrapped vtkvmtk extension modules load (which exercises the
    linkage against the VTK libraries of the "vtk" wheel),
  * the ITK-based segmentation classes work (which exercises the ITK
    libraries, statically linked on Windows and bundled on Linux/macOS),
  * the pype mechanism and the vmtk script modules import.

No rendering window is created, so the test also runs on headless CI
machines.
"""

import sys


def test_import():
    import vmtk
    from vmtk import vtkvmtk
    print("vmtk package:", vmtk.__file__)

    # One class from each of the main wrapped kits.
    for class_name in [
        "vtkvmtkCenterlineSmoothing",       # ComputationalGeometry
        "vtkvmtkPolyDataSurfaceRemeshing",  # Misc
        "vtkvmtkCapPolyData",               # ComputationalGeometry/Misc
        "vtkvmtkGradientMagnitudeImageFilter",  # Segmentation (ITK-based)
    ]:
        cls = getattr(vtkvmtk, class_name, None)
        if cls is None:
            # Not all classes exist in all configurations; require at least
            # the ITK-based one below.
            print("note: class not found:", class_name)
            continue
        instance = cls()
        print("instantiated", instance.GetClassName())

    # TetGen is excluded from PyPI packages for license reasons (see
    # distribution/pypi/README.md); make sure it did not sneak in.
    import os
    if os.getenv("VMTK_BUILD_TETGEN", "OFF").upper() not in ("ON", "1", "TRUE"):
        assert not hasattr(vtkvmtk, "vtkvmtkTetGenWrapper"), (
            "vtkvmtkTetGenWrapper found in the wheel although TetGen "
            "must not be distributed on PyPI (license terms)"
        )
        print("TetGen correctly excluded")


def test_itk_filter():
    """Run an ITK-based filter end to end on a small synthetic image."""
    import vtk
    from vmtk import vtkvmtk

    source = vtk.vtkImageEllipsoidSource()
    source.SetWholeExtent(0, 15, 0, 15, 0, 15)
    source.SetCenter(8, 8, 8)
    source.SetRadius(4, 4, 4)
    source.SetInValue(100.0)
    source.SetOutValue(0.0)
    source.SetOutputScalarTypeToFloat()
    source.Update()

    gradient = vtkvmtk.vtkvmtkGradientMagnitudeImageFilter()
    gradient.SetInputConnection(source.GetOutputPort())
    gradient.Update()

    scalar_range = gradient.GetOutput().GetScalarRange()
    print("gradient magnitude range:", scalar_range)
    assert scalar_range[1] > 0.0, "ITK gradient magnitude produced no output"


def test_pypes():
    from vmtk import pypes, pypemain, vmtkscripts  # noqa: F401

    pipe = pypes.Pype()
    pipe.ExitOnError = 0
    pipe.Arguments = ["vmtkimagereader", "--help"]
    pipe.ParseArguments()
    print("pype parsed arguments OK")

    # joblib is an optional dependency: vmtkcenterlinesnetwork must import
    # and construct with or without it (serial fallback).
    from vmtk import vmtkcenterlinesnetwork
    script = vmtkcenterlinesnetwork.vmtkCenterlinesNetwork()
    try:
        import joblib  # noqa: F401
        have_joblib = True
    except ImportError:
        have_joblib = False
    print(f"vmtkcenterlinesnetwork OK (joblib installed: {have_joblib}, "
          f"UseJoblib: {bool(script.UseJoblib)})")


def main():
    test_import()
    test_itk_filter()
    test_pypes()
    print("vmtk wheel smoke test passed")
    return 0


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkmeshviewer.py,v $
## Language:  Python
## Date:      $Date: 2006/05/26 12:35:13 $
## Version:   $Revision: 1.9 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import vmtkrenderer
from vmtk import pypes


class vmtkMeshViewer(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Mesh = None
        self.vmtkRenderer = None
        self.OwnRenderer = 0
        self.Display = 1
        self.Opacity = 1.0
        self.ArrayName = ''
        self.ScalarRange = [0.0, 0.0]
        self.ColorMap = 'cooltowarm'
        self.NumberOfColors = 256
        self.Color = [-1.0, -1.0, -1.0]
        self.Legend = 0
        self.FlatInterpolation = 0

        self.Representation = 'surface'

        self.Actor = None
        self.ScalarBarActor = None

        self.SetScriptName('vmtkmeshviewer')
        self.SetScriptDoc('display a mesh')
        self.SetInputMembers([
            ['Mesh','i','vtkUnstructuredGrid',1,'','the input mesh','vmtkmeshreader'],
            ['vmtkRenderer','renderer','vmtkRenderer',1,'','external renderer'],
            ['Display','display','bool',1,'','toggle rendering'],
            ['Opacity','opacity','float',1,'(0.0,1.0)','object opacity in the scene'],
            ['ArrayName','array','str',1,'','name of the array where the scalars to be displayed are stored'],
            ['ScalarRange','scalarrange','float',2,'','range of the scalar map'],
            ['ColorMap','colormap','str',1,'["rainbow","blackbody","cooltowarm","grayscale"]','choose the color map'],
            ['NumberOfColors','numberofcolors','int',1,'','number of colors in the color map'],
            ['Legend','legend','bool',1,'','toggle scalar bar'],
            ['Color','color','float',3,'','RGB color of the object in the scene'],
            ['FlatInterpolation','flat','bool',1,'','toggle flat or shaded surface display']
            ])
        self.SetOutputMembers([
            ['Mesh','o','vtkUnstructuredGrid',1,'','the output mesh','vmtkmeshwriter']
            ])

    def RepresentationCallback(self, obj):
        if not self.Actor:
            return

        if self.Representation == 'surface':
            self.Representation = 'edges'
        elif self.Representation == 'edges':
            self.Representation = 'wireframe'
        elif self.Representation == 'wireframe':
            self.Representation = 'surface'

        if self.Representation == 'surface':
            self.Actor.GetProperty().SetRepresentationToSurface()
            self.Actor.GetProperty().EdgeVisibilityOff()
        elif self.Representation == 'edges':
            self.Actor.GetProperty().SetRepresentationToSurface()
            self.Actor.GetProperty().EdgeVisibilityOn()
        elif self.Representation == 'wireframe':
            self.Actor.GetProperty().SetRepresentationToWireframe()
            self.Actor.GetProperty().EdgeVisibilityOff()

        self.vmtkRenderer.RenderWindow.Render()

    def BuildView(self):

        if not self.vmtkRenderer:
            self.vmtkRenderer = vmtkrenderer.vmtkRenderer()
            self.vmtkRenderer.Initialize()
            self.OwnRenderer = 1

        self.vmtkRenderer.RegisterScript(self)

        if self.Actor:
            self.vmtkRenderer.Renderer.RemoveActor(self.Actor)

        if self.ScalarBarActor:
            self.vmtkRenderer.Renderer.RemoveActor(self.ScalarBarActor)

        if self.Mesh:
            mapper = vtk.vtkDataSetMapper()
            mapper.SetInputData(self.Mesh)
            if self.ArrayName:
                self.Mesh.GetPointData().SetActiveScalars(self.ArrayName)
                array = self.Mesh.GetPointData().GetScalars()

                if self.ScalarRange[1] > self.ScalarRange[0]:
                    mapper.SetScalarRange(self.ScalarRange)
                else:
                    mapper.SetScalarRange(array.GetRange(0))

                if self.ColorMap == 'grayscale':
                    lut = mapper.GetLookupTable()
                    lut.SetNumberOfTableValues(self.NumberOfColors)
                    lut.SetValueRange(0.0,1.0)
                    lut.SetSaturationRange(0.0,0.0)
                    lut.Build()
                    mapper.SetLookupTable(lut)

                if self.ColorMap == 'rainbow':
                    lut = mapper.GetLookupTable()
                    lut.SetHueRange(0.666667,0.0)
                    lut.SetSaturationRange(0.75,0.75)
                    lut.SetValueRange(1.0,1.0)
                    lut.SetAlphaRange(1.0,1.0)
                    lut.SetNumberOfColors(self.NumberOfColors)
                    lut.Build()
                    mapper.SetLookupTable(lut)

                if self.ColorMap == 'blackbody':
                   lut = mapper.GetLookupTable()
                   lut.SetNumberOfTableValues(self.NumberOfColors)
                   colorTransferFunction = vtk.vtkColorTransferFunction()
                   colorTransferFunction.SetColorSpaceToRGB()
                   colorTransferFunction.AddRGBPoint(0,0.0,0.0,0.0)
                   colorTransferFunction.AddRGBPoint(0.4,0.901961,0.0,0.0)
                   colorTransferFunction.AddRGBPoint(0.8,0.901961,0.901961,0.0)
                   colorTransferFunction.AddRGBPoint(1.0,1.0,1.0,1.0)
                   for ii,ss in enumerate([float(xx)/float(self.NumberOfColors) for xx in range(self.NumberOfColors)]):
                       cc = colorTransferFunction.GetColor(ss)
                       lut.SetTableValue(ii,cc[0],cc[1],cc[2],1.0)
                   lut.Build()
                   mapper.SetLookupTable(lut)

                if self.ColorMap == 'cooltowarm':
                   lut = mapper.GetLookupTable()
                   lut.SetNumberOfTableValues(self.NumberOfColors)
                   colorTransferFunction = vtk.vtkColorTransferFunction()
                   colorTransferFunction.SetColorSpaceToDiverging()
                   colorTransferFunction.AddRGBPoint(0,0.231373,0.298039,0.752941)
                   colorTransferFunction.AddRGBPoint(0.5,0.865003,0.865003,0.865003)
                   colorTransferFunction.AddRGBPoint(1.0,0.705882,0.0156863,0.14902)
                   for ii,ss in enumerate([float(xx)/float(self.NumberOfColors) for xx in range(self.NumberOfColors)]):
                       cc = colorTransferFunction.GetColor(ss)
                       lut.SetTableValue(ii,cc[0],cc[1],cc[2],1.0)
                   lut.Build()
                   mapper.SetLookupTable(lut)
            else:
                mapper.ScalarVisibilityOff()
            self.Actor = vtk.vtkActor()
            self.Actor.SetMapper(mapper)
            if (self.Color[0] >= 0.0):
                self.Actor.GetProperty().SetColor(self.Color)
            if self.FlatInterpolation:
                self.Actor.GetProperty().SetInterpolationToFlat()
            self.Actor.GetProperty().SetOpacity(self.Opacity)
            self.vmtkRenderer.Renderer.AddActor(self.Actor)
            self.vmtkRenderer.AddKeyBinding('w','Change surface representation.',self.RepresentationCallback)

        if self.Legend and self.Actor:
            self.ScalarBarActor = vtk.vtkScalarBarActor()
            self.ScalarBarActor.SetLookupTable(self.Actor.GetMapper().GetLookupTable())
            self.ScalarBarActor.GetLabelTextProperty().ItalicOff()
            self.ScalarBarActor.GetLabelTextProperty().BoldOff()
            self.ScalarBarActor.GetLabelTextProperty().ShadowOff()
##             self.ScalarBarActor.GetLabelTextProperty().SetColor(0.0,0.0,0.0)
            self.ScalarBarActor.SetLabelFormat('%.2f')
            self.vmtkRenderer.Renderer.AddActor(self.ScalarBarActor)

        if self.Display:
            self.vmtkRenderer.Render()

        if self.OwnRenderer:
            self.vmtkRenderer.Deallocate()

    def Execute(self):

        if (self.Mesh == None) & (self.Display == 1):
            self.PrintError('Error: no Mesh.')

        self.BuildView()


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()
